/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.engine.cache;

import com.funambol.email.items.model.SendResult;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.util.ArrayList;
import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;

import com.funambol.framework.tools.DBTools;

import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;

/**
 * Contains methods to access to the table with the state of the items.
 *
 * @version $Id: CrcItemStateDAO.java,v 1.5 2006/08/08 09:29:36 gibi Exp $
 *
 */
public class CrcItemStateDAO {

    // --------------------------------------------------------------- Constants

    private static final String SQL_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE =
        "select id, last_crc from fnbl_email_cache " +
        " where source_uri = ? and principal = ?";

    private static final String SQL_ADD_LOCAL_ITEM  =
        "insert into fnbl_email_cache " +
        "(id, last_crc, source_uri, principal) values(?, ?, ?, ?)"                                                   ;

    private static final String SQL_DELETE_LOCAL_ITEMS  =
        "delete from fnbl_email_cache " +
        " where source_uri = ? and principal = ? ";

    private static final String SQL_DELETE_POP  =
        "delete from fnbl_email_sentpop " +
        " where source_uri = ? and principal = ?";

    private Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // ------------------------------------------------------------ Private data

    /**         */
    private String jndiDataSourceName;

    /**         */
    private DataSource dataSource;


    // ------------------------------------------------------------- Constructor

    /**
     *
     */
    private CrcItemStateDAO() {
    }

    /**
     *
     *
     * @param jndiDataSourceName String
     */
    public CrcItemStateDAO(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }


    // ---------------------------------------------------------- Public Methods

    /**
     *
     * @throws Exception
     */
    public void init() throws Exception {
        lookupDataSource();
    }

    /**
     *
     *
     * @return String
     */
    public String getJndiDataSourceName() {
        return jndiDataSourceName;
    }

    /**
     *
     * @param jndiDataSourceName String
     */
    public void setJndiDataSourceName(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }

    /**
     * Adds local items
     *
     *
     * @param items CrcSyncItemInfo[]
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    public void addLocalItems(CrcSyncItemInfo[] items,
                              ArrayList itemsSent,
                              String sourceURI,
                              long principal) throws Exception {

         SendResult         sr            ;
         Connection         connection    = null ;
         PreparedStatement  ps            = null ;
         String             id            = null;
         long               lastCrc       ;
         String     GUID;
         String     LUID;
         boolean    isSent;
         boolean    isPOP;

         try {

            connection = dataSource.getConnection();

            if (log.isLoggable(Level.FINEST)) {
               log.finest("CrcItemStateDAO add start " + items.length);
            }

            int itemsLen = items.length;
            int sentLen  = itemsSent.size();

            for (int i=0, l = itemsLen; i < l; i++) {

                ps = connection.prepareStatement(SQL_ADD_LOCAL_ITEM);

                id       = items[i].getKey().getKeyAsString();
                lastCrc  = items[i].getLastCrc() ;

                if (log.isLoggable(Level.FINEST)) {
                   log.finest("   CrcItemStateDAO add " + id + " - " + lastCrc);
                }

                if (id.startsWith(Def.FOLDER_SENT_ID)){
                    for (int y=0; y<sentLen; y++) {
                        sr = (SendResult) itemsSent.get(y);
                        GUID   = sr.getGUID();
                        isSent = sr.isSent();
                        if (GUID.equals(id) && isSent){
                            id = Utility.convertSentIDToOutboxID(id);
                            break;
                        }
                    }
                }

                ps.setString (1, id        ) ;
                ps.setLong   (2, lastCrc   ) ;
                ps.setString (3, sourceURI ) ;
                ps.setLong   (4, principal ) ;

                ps.executeUpdate();

                ps.close() ;
                ps = null  ;
            }
         } catch (Exception e) {
             throw new Exception("Error adding local items", e);
         } finally {
             DBTools.close(connection, ps, null);
         }
    }


    /**
     * Delete local items
     *
     * @param sourceURI
     * @param principalId
     *
     * @throws Exception
     **/
    public void deleteSentPop(String sourceURI, long principalId)
        throws Exception {

        Connection         connection  = null ;
        PreparedStatement  ps          = null ;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcItemStateDAO delete fnbl_email_sentpop "
                       + principalId+ " - " +sourceURI);
        }
        try {

            connection = dataSource.getConnection();

            ps = connection.prepareStatement(SQL_DELETE_POP);

            ps.setString  (1, sourceURI ) ;
            ps.setLong    (2, principalId ) ;

            ps.executeUpdate();

        } catch (Exception e) {
            throw new Exception("Error deleting fnbl_email_sentpop", e);
        } finally {
            DBTools.close(connection, ps, null);
        }
    }

    /**
     * Delete local items
     *
     *
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    public void deleteLocalItems(String sourceURI, long principal)
        throws Exception {

        Connection         connection  = null ;
        PreparedStatement  ps          = null ;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcItemStateDAO delete start "+ principal+ " - " +sourceURI);
        }
        try {

            connection = dataSource.getConnection();

            ps = connection.prepareStatement(SQL_DELETE_LOCAL_ITEMS);

            ps.setString  (1, sourceURI ) ;
            ps.setLong    (2, principal ) ;

            ps.executeUpdate();

        } catch (Exception e) {
            throw new Exception("Error deleting local items", e);
        } finally {
            DBTools.close(connection, ps, null);
        }
    }

    /**
     * Update local items
     *
     *
     * @param localItems CrcSyncItemInfo[]
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    public void updateLocalItems(CrcSyncItemInfo[] items ,
                                 ArrayList itemsSent,
                                 String sourceURI,
                                 long principal)
    throws Exception {

         if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcItemStateDAO update start");
         }
         this.deleteLocalItems (sourceURI, principal);
         this.addLocalItems    (items, itemsSent, sourceURI, principal);
         if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcItemStateDAO update end");
         }
    }

    /**
     * Gets local items
     *
     *
     * @param sourceURI String
     * @param principalId String
     * @return CrcSyncItemInfo[]
     * @throws Exception
     */
    public CrcSyncItemInfo[] getLocalItems (String sourceURI,
                                            long principalId)
        throws Exception {

         Connection connection   = null;
         PreparedStatement ps    = null;
         ResultSet rs            = null;
         ArrayList rows          = null;
         CrcSyncItemInfo[] items = null;

         try {

             connection = dataSource.getConnection();

             rows = new ArrayList();

             ps = connection.prepareStatement(
                    SQL_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE);

             ps.setString(1, sourceURI);
             ps.setLong(2, principalId);

             rs = ps.executeQuery();

             while(rs.next()) {
                 rows.add(new CrcSyncItemInfo(rs.getString(1), rs.getLong(2)));
             }

             items = (CrcSyncItemInfo[]) rows.toArray(new CrcSyncItemInfo[0]);

         } catch (Exception e) {
             throw new Exception("Error getting local items", e);
         } finally {
             DBTools.close(connection, ps, rs);
         }

         return items;
    }



    // --------------------------------------------------------- Private Methods

    /**
     * Lookups the datasource
     *
     * JBOSS
     * private String jndiDataSourceName = "java:jdbc/sync4j";
     *
     * TOMCAT
     * private String jndiDataSourceName = "java:comp/env/jdbc/sync4j";
     *
     * @throws Exception if a error occurs during lookup
     */
    private void lookupDataSource() throws Exception {

        InitialContext ctx = new InitialContext();
        if (log.isLoggable(Level.FINEST)) {
              log.finest("CachedSyncSource: datasource lookup");
        }

        try {
          // try tomcat  i.e. java:comp/env/jdbc/sync4j
          dataSource = (DataSource) ctx.lookup("java:comp/env/"+jndiDataSourceName) ;
          if (log.isLoggable(Level.FINEST)) {
             log.finest("CachedSyncSource: datasource lookup done using java:comp/env/ prefix");
          }
        }
        catch (NamingException ne) {
          if (log.isLoggable(Level.FINEST)) {
              log.finest("CachedSyncSource: No tomcat datasource lookup!");
          }
          try {
              // try jboss i.e. java:jdbc/sync4j
              dataSource = (DataSource) ctx.lookup("java:"+jndiDataSourceName) ;
              if (log.isLoggable(Level.FINEST)) {
                 log.finest("CachedSyncSource: datasource lookup done using java: prefix");
              }
          }
          catch (Exception ex){
            if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: No jboss datasource lookup!");
            }
            try {
                // try othser j2ee server jdbc/sync4j
                dataSource = (DataSource) ctx.lookup(""+jndiDataSourceName) ;
                if (log.isLoggable(Level.FINEST)) {
                   log.finest("CachedSyncSource: datasource lookup done using - prefix");
                }
            }
            catch (Exception e){
              if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: No j2ee server datasource lookup!");
              }
              throw new Exception("Data source "+ jndiDataSourceName
                                  + " not found " + e.toString());
            }
          }
        }
    }

}
