/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.engine.source;

import java.io.Serializable;
import java.util.logging.Logger;
import java.util.ArrayList;
import java.util.logging.Level;
import java.sql.Timestamp;

import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.engine.source.FilterableSyncSource;
import com.funambol.framework.engine.source.SyncSourceException;
import com.funambol.framework.engine.SyncItemState;
import com.funambol.framework.tools.beans.LazyInitBean;
import com.funambol.framework.tools.beans.BeanInitializationException;
import com.funambol.framework.security.Sync4jPrincipal;

import com.funambol.email.pdi.mail.*;
import com.funambol.email.exception.EntityException;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.items.manager.ImapEntityManager;
import com.funambol.email.items.model.SendResult;
import com.funambol.email.engine.cache.CrcSyncItemInfo;
import com.funambol.email.exception.SendingException;
import com.funambol.email.pdi.folder.Folder;
import com.funambol.framework.engine.source.SyncContext;


/**
 * This class implements common methods of EmailIMAPSyncSource
 * <p/>
 * version $Id: ImapEmailSyncSource.java,v 1.2 2006/08/08 11:03:58 gibi Exp $
 */
public final class ImapEmailSyncSource
        extends EmailSyncSource
        implements FilterableSyncSource, Serializable, LazyInitBean {
    // --------------------------------------------------------------- Constants

    /**  */
    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // -------------------------------------------------------------- Properties

    /**  */
    protected ImapEntityManager iem = null;

    {
        this.setInPort("" + Def.PROTOCOL_IMAP_PORT);
    }

    // ---------------------------------------------------------- Public Methods


    /**
     *
     */
    public void beginSync(SyncContext context) throws SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("IMAP EmailSyncSource - Start beginSync [" + sourceURI + "]");
        }
        super.beginSync(context);
    }


    /**
     * Init the syncSource
     *
     * @throws BeanInitializationException
     */
    public void init() throws BeanInitializationException {
        super.init();
    }


    /**
     * @param syncItem SyncItem
     * @return boolean
     * @throws SyncSourceException
     */
    public boolean isSyncItemInFilterClause(SyncItem syncItem)
      throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("IMAP isSyncItemInFilterClause(syncItem)");
        }

        String content;
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " +
                    syncItem, e);
        }

        boolean isemail = isEmailFromContent(content);

        boolean isInFilter = false;

        if (isemail) {
            isInFilter = isEmailInFilter(content, syncItem);
        } else {
            isInFilter = isFolderInFilter(content, syncItem);
        }

        return isInFilter;
    }

    /**
     * @param syncItemKey SyncItemKey
     * @return boolean
     * @throws SyncSourceException
     */
    public boolean isSyncItemInFilterClause(SyncItemKey syncItemKey)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("IMAP isSyncItemInFilterClause(syncItemKey)");
        }

        String key  = (String) syncItemKey.getKeyValue();
        String FID  = Utility.getIdBeforeSeparator(key);
        String FMID = Utility.getIdAfterSeparator(key);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("isSyncItemInFilter in FID: " + FID + " with FMID: " + FMID);
        }

        boolean isemail = isEmailFromKey(FID, FMID);

        boolean isInFilter = false;

        if (isemail) {
            isInFilter = isEmailInFilter(key);
        } else {
            isInFilter = isFolderInFilter(key);
        }

        return isInFilter;
    }


    /**
     * @param operation  String
     * @param statusCode int
     * @param keys       SyncItemKey[]
     */
    public void setOperationStatus(String operation,
                                   int statusCode,
                                   SyncItemKey[] keys) {

        if (log.isLoggable(Level.FINEST)) {
            StringBuffer message = new StringBuffer("Received status code '");
            message.append(statusCode).append("' for a '").append(operation).append("'").
                    append(" for this items: ");

            for (int i = 0; i < keys.length; i++) {
                message.append("\n- ").append(keys[i].getKeyAsString());
            }
            log.finest(message.toString());
        }
    }


    /**
     * @param syncItemKey SyncItemKey
     * @param s           Timestamp
     * @param softDelete  boolean
     * @throws SyncSourceException
     */
    public void removeSyncItem(SyncItemKey syncItemKey,
                               Timestamp s,
                               boolean softDelete)
            throws SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("remove item softDelete: " + softDelete);
        }

        if (softDelete) {
            // nothing
        } else {
            // if false
            removeItem(syncItemKey);
        }

    }


    /**
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem getSyncItemFromId(SyncItemKey syncItemKey)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemFromId start");
        }

        SyncItem syncItem;
        String key = (String) syncItemKey.getKeyValue();

        String FID = Utility.getIdBeforeSeparator(key);
        String FMID = Utility.getIdAfterSeparator(key);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("get item in FID: " + FID + " with FMID: " + FMID);
        }

        boolean check = isEmailFromKey(FID, FMID);

        if (check) {
            syncItem = getEmailFromId(syncItemKey);
        } else {
            syncItem = getFolderFromId(syncItemKey);
        }

        return syncItem;

    }


    /**
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getAllSyncItemKeys() throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItems start ");
        }

        String[]   allIds;
        String[]   mailIds;
        String[]   folderIds;

        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {

            this.iem = getImapEntityManager();

            mailIds = iem.getAllEmails(filter, this.allMailboxActivation,
                                       this.sourceURI, pID);

            folderIds = iem.getAllFolders(filter, this.allMailboxActivation,
                                          this.sourceURI,  pID);
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting manager layer: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error reading getAllSyncItems : " +
                    e.getMessage(), e);
        }

        // put the list togheter
        allIds = Utility.mergeArray(mailIds, folderIds, null, null, null);


        if (log.isLoggable(Level.FINEST)) {
            log.finest("Imap getAllSyncItems() end");
        }

        SyncItemKey[] syncItemKeys = null;
        if (allIds != null) {
            // remove the GUID null (this is the case of the new folders)
            ArrayList onlyGoodIds = new ArrayList();
            String key;
            for (int i = 0; i < allIds.length; i++) {
                key = allIds[i];
                if (key != null) {
                    onlyGoodIds.add(key);
                }
            }
            int numfin = onlyGoodIds.size();
            // create the syncItemKeys without null
            syncItemKeys = new SyncItemKey[numfin];
            for (int i = 0; i < numfin; i++) {
                syncItemKeys[i] = new SyncItemKey(allIds[i]);
            }
        }

        return syncItemKeys;
    }


    /**
     * @return CrcSyncItemInfo[]
     * @throws SyncSourceException
     */
    public CrcSyncItemInfo[] getAllSyncItemInfo()
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo start");
        }

        CrcSyncItemInfo[] allInfo;
        //
        CrcSyncItemInfo[] emailInfo;
        CrcSyncItemInfo[] folderInfo;


        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {

            this.iem = getImapEntityManager();

            emailInfo = this.iem.getAllEmailsInfo(filter, this.allMailboxActivation,
                                                  this.sourceURI, pID);

            folderInfo = this.iem.getAllFoldersInfo(filter, this.allMailboxActivation,
                                                    this.sourceURI, pID);

        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting manager layer: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error reading emails/folders: " +
                    e.getMessage(), e);
        }

        // add main folders
        allInfo = Utility.mergeArray(emailInfo, folderInfo, null, null, null);

        return allInfo;
    }

    /**
     * @throws SyncSourceException
     */
    public void clearServerInfo() throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("cleanServerInfo start");
        }

        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {

            this.iem = getImapEntityManager();

            this.iem.clearAllItems(this.allMailboxActivation,
                    this.sourceURI,
                    pID);

        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting manager layer: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error deleting all emails/folders: " +
                    e.getMessage(), e);
        }


    }

    /**
     * @param syncItem SyncItem
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getSyncItemKeysFromTwin(SyncItem syncItem)
            throws SyncSourceException {

        String content;
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " +
                    syncItem, e);
        }

        boolean isemail = isEmailFromContent(content);

        SyncItemKey[] syncItemKeys;

        if (isemail) {
            syncItemKeys = getEmailFromTwin(content, syncItem);
        } else {
            syncItemKeys = getFolderFromTwin(content, syncItem);
        }

        return syncItemKeys;
    }

    /**
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem addSyncItem(SyncItem syncItem) throws SyncSourceException {

        String content;
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " + syncItem, e);
        }
        boolean isEmail = isEmailFromContent(content);
        if (isEmail) {
            syncItem = addEmail(content, syncItem);
        } else {
            syncItem = addFolder(content, syncItem);
        }
        return syncItem;
    }


    /**
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem updateSyncItem(SyncItem syncItem) throws SyncSourceException {

        String content;
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " + syncItem, e);
        }

        boolean isEmail = isEmailFromContent(content);

        String key = syncItem.getKey().getKeyAsString();

        String parentId = Utility.getIdBeforeSeparator(key);
        String objectId = Utility.getIdAfterSeparator(key);

        if (isEmail) {
            syncItem = updateEmail(parentId, objectId, content, syncItem);
        } else {
            syncItem = updateFolder(parentId, objectId, content, syncItem);
        }

        return syncItem;

    }

    // --------------------------------------------------------- Private Methods

    /**
     * Create EmailIMAPManager istance (if not already exist)
     *
     * @return EmailIMAPManager
     * @throws EmailAccessException
     */
    private ImapEntityManager getImapEntityManager() throws EmailAccessException {
        try {
            if (this.iem == null) {
                this.iem = new ImapEntityManager(this.mswf, this.jndiDataSourceName);
            }
        }
        catch (Exception e) {
            throw new EmailAccessException(e.getMessage());
        }
        return this.iem;
    }

    /**
     * @param syncItemKey SyncItemKey
     * @throws SyncSourceException
     */
    private void removeItem(SyncItemKey syncItemKey)
            throws SyncSourceException {

        String key = (String) syncItemKey.getKeyValue();

        String parentId = Utility.getIdBeforeSeparator(key);
        String objectId = Utility.getIdAfterSeparator(key);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("remove item in FID: " + parentId +
                    " with FMID: " + objectId);
        }

        boolean isEmail = isEmailFromKey(parentId, objectId);

        if (isEmail) {
            removeEmail(parentId, objectId);
        } else {
            removeFolder(parentId, objectId);
        }

    }

    /**
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem addEmail(String content, SyncItem syncItem)
      throws SyncSourceException {

        String[] keys = Utility.getIdBeforeAndAfter(syncItem);
        String FID    = keys[0];
        String FMID   = keys[1];

        if (log.isLoggable(Level.FINEST)) {
            log.finest("addEmail in folder: " + FID + " luid: " + FMID);
        }

        Email  ctmpNew       = null;
        Email  insertedEmail = null;
        String GUID          = null;
        long   pID           = ((Sync4jPrincipal) this.principal).getId();

        try {
            ctmpNew = getFoundationMailFromXML(content);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        try {

            this.iem = getImapEntityManager();

            // add and send the mail
            insertedEmail = this.iem.addEmail(FID, FMID, ctmpNew,
                                              this.sourceURI, pID);

            // message add and sent without error, set to true the result
            if (insertedEmail != null) {
                GUID = (String) insertedEmail.getUID().getPropertyValue();
                String idtmp = Utility.convertOutboxIDToSentID(GUID);
                SendResult sr = new SendResult(idtmp, FMID, true, false);
                itemsSent.add(sr);
            }


        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting manager layer: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new SyncSourceException(errorMessage, e);
        } catch (SendingException e) {
            SendResult sr = new SendResult(GUID, FMID, false, false);
            itemsSent.add(sr);
            throw new SyncSourceException(e.getMessage(), 500);
        }


        // create the output syncItem
        try {
            if (insertedEmail != null) {
                syncItem = createSyncItem(insertedEmail, SyncItemState.NEW);
            }
        } catch (Exception e) {
            throw new SyncSourceException("Error creating SyncItem: " +  syncItem, e);
        }

        return syncItem;

    }


    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isEmailInFilter(String key) throws SyncSourceException {

        boolean isInFilter = false;

        String parentId = Utility.getIdBeforeSeparator(key);
        String mailId   = Utility.getIdAfterSeparator(key);

        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {
            this.iem   = getImapEntityManager();
            isInFilter = this.iem.isEmailInFilter(filter, key,
                                                  parentId, mailId,
                                                  this.sourceURI, pID);
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting manager layer: " +
                                          e.getMessage(), e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error checking the item: " +
                                          e.getMessage() , e);
        }


        return isInFilter;

    }

    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isEmailInFilter(String content, SyncItem syncItem)
      throws SyncSourceException {

        boolean isInFilter = false;

        // get SyncItem UID
        String key    = syncItem.getKey().getKeyAsString();
        String[] keys = Utility.getIdBeforeAndAfter(syncItem);
        String FID    = keys[0];
        String FMID   = keys[1];

        if (log.isLoggable(Level.FINEST)) {
            log.finest("isEmailInFilter with UID: " + key +
                       " FMID " + FMID + " FID " + FID);
        }

        long pID = ((Sync4jPrincipal) this.principal).getId();

        Email email;
        try {

            email = getFoundationMailFromXML(content);

            this.iem = getImapEntityManager();

            isInFilter = this.iem.isEmailInFilter(filter, FID, key, email);

        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error accessing MailBox", e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error matching mail with filter ", e);
        }

        return isInFilter;

    }

    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isFolderInFilter(String key) throws SyncSourceException {
        boolean isInFilter = false;
        return isInFilter;

    }

    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isFolderInFilter(String content, SyncItem syncItem)
     throws SyncSourceException {
        boolean isInFilter = false;
        return isInFilter;

    }

    /**
     * Before inserting the folder it checks if the folder
     * already exists. This step is necessary because the
     * "twin" method returns always "item not found"
     * <p/>
     * the server in the slow sync will always call the Add method
     * (there is no filter provided by the Twin Methods)
     * <p/>
     * The add method must choose between
     * 1) the folder already exists (returns the old GUID)
     * 2) the folder must be created (returns the new GUID)
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem addFolder(String content, SyncItem syncItem)
            throws SyncSourceException {

        Folder ctmpNew;
        try {
            ctmpNew = getFoundationFolderFromXML(content);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        Folder insertedFolder = null;
        String GUID;

        String name     = ctmpNew.getName().getPropertyValueAsString();
        String parentId = syncItem.getParentKey().getKeyAsString();

        if (parentId.equalsIgnoreCase("/")) {
            parentId = Def.FOLDER_ROOT_ID;
        }

        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {

            this.iem = getImapEntityManager();

            // check if the folder already exists
            GUID = this.iem.getFolderFromName(name, parentId, this.sourceURI, pID);

            // if null I have to insert the folder otherwise
            // the folder already exists
            if (GUID == null) {
                insertedFolder = this.iem.addFolder(name, parentId,
                                                    this.defaultFolder,
                                                    idFolderSpace,
                                                    this.sourceURI, pID);
            }

        } catch (Exception e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new SyncSourceException(errorMessage, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("End insert folder. start create SyncItem");
        }

        // create the output syncItem
        try {
            if (insertedFolder != null) {
                syncItem = createSyncItem(insertedFolder, SyncItemState.NEW);
            }
        } catch (Exception e) {
            throw new SyncSourceException("Error creating SyncItem: " +
                    syncItem, e);
        }

        return syncItem;
    }

    /**
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem getEmailFromId(SyncItemKey syncItemKey)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getEmailFromId start");
        }

        SyncItem syncItem = null;
        Email    mail     = null;

        String key      = syncItemKey.getKeyAsString();
        String parentId = Utility.getIdBeforeSeparator(key);
        String mailId   = Utility.getIdAfterSeparator(key);

        long pID        = ((Sync4jPrincipal) this.principal).getId();

        char status     = getSyncItemStateFromId(syncItemKey);

        try {
            this.iem = getImapEntityManager();
            mail = this.iem.getEmailFromUID(filter, mailId, parentId, status,
                                            this.sourceURI, pID);
        } catch (Exception e) {
            throw new SyncSourceException("Error in getting Email ", e);
        }

        if (mail != null) {
            try {
                syncItem = createSyncItem(mail, status);
            } catch (Exception e) {
                throw new SyncSourceException("Error in creating SyncItem ", e);
            }
        }

        return syncItem;

    }

    /**
     * the folder syncItemKey is the GUID in the
     * local DB.
     *
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem getFolderFromId(SyncItemKey syncItemKey)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getFolderFromId start");
        }

        SyncItem syncItem = null;
        Folder   folder   = null;

        String GUID = syncItemKey.getKeyAsString();
        long   pID  = ((Sync4jPrincipal) this.principal).getId();

        try {
            this.iem = getImapEntityManager();
            folder = this.iem.getFolderFromUID(filter, GUID, this.sourceURI, pID);
        } catch (Exception e) {
            throw new SyncSourceException("Error in getting Folder ", e);
        }

        // get item status
        char status = getSyncItemStateFromId(syncItemKey);

        if (folder != null) {
            try {
                syncItem = createSyncItem(folder, status);
            } catch (Exception e) {
                throw new SyncSourceException("Error in creating SyncItem ", e);
            }
        }

        return syncItem;

    }

    /**
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    private SyncItemKey[] getEmailFromTwin(String content,
                                           SyncItem syncItem)
            throws SyncSourceException {

        // get SyncItem UID
        String key = syncItem.getKey().getKeyAsString();

        String[] keys = Utility.getIdBeforeAndAfter(syncItem);
        String FID = keys[0];
        String FMID = keys[1];

        String GUID;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getEmailFromTwin with UID: " + key +
                    " FMID " + FMID + " FID " + FID);
        }

        Email email;
        try {
            email = getFoundationMailFromXML(content);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {
            this.iem = getImapEntityManager();
            GUID = this.iem.getEmailFromClause(FID, FMID, email, this.sourceURI, pID);
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error accessing MailBox", e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error getting twin", e);
        }

        SyncItemKey[] syncItemKeys;
        if (GUID != null) {
            syncItemKeys = new SyncItemKey[1];
            syncItemKeys[0] = new SyncItemKey(GUID);
        } else {
            syncItemKeys = new SyncItemKey[0];
        }

        return syncItemKeys;
    }

    /**
     * this method return always "twin not found"
     * the server will always call the Add method
     * The add method must choose between
     * 1) the folder already exists (returns the old GUID)
     * 2) the folder must be created (returns the new GUID)
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    private SyncItemKey[] getFolderFromTwin(String content,
                                            SyncItem syncItem)
            throws SyncSourceException {


        SyncItemKey[] syncItemKeys;
        String GUID;

        String key = (String) syncItem.getKey().getKeyValue();
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemKeysFromTwin with UID: " + key);
        }
        Folder ctmpNew;
        try {
            ctmpNew = getFoundationFolderFromXML(content);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        String name = ctmpNew.getName().getPropertyValueAsString();
        String parentId = syncItem.getParentKey().getKeyAsString();

        if (parentId.equalsIgnoreCase("/")) {
            parentId = Def.FOLDER_ROOT_ID;
        }

        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {

            this.iem = getImapEntityManager();

            // check if the folder already exists
            GUID = this.iem.getFolderFromName(name, parentId, this.sourceURI, pID);

            if (GUID != null) {
                syncItemKeys = new SyncItemKey [1];
                syncItemKeys[0] = new SyncItemKey(GUID);
            } else {
                syncItemKeys = new SyncItemKey[0];
            }


        } catch (Exception e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new SyncSourceException(errorMessage, e);
        }

        return syncItemKeys;
    }

    /**
     * @param parentId String
     * @param mailId   mail id in the folder not unique
     * @throws SyncSourceException
     */
    private void removeEmail(String parentId, String mailId)
            throws SyncSourceException {

        long pID = ((Sync4jPrincipal) this.principal).getId();
        try {
            this.iem = getImapEntityManager();
            this.iem.removeEmail(parentId, mailId, this.sourceURI, pID);
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error accessing MailBox", e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error removing email", e);
        }
    }

    /**
     * @param parentId String
     * @param folderId String
     * @throws SyncSourceException
     */
    private void removeFolder(String parentId, String folderId)
            throws SyncSourceException {

        long pID = ((Sync4jPrincipal) this.principal).getId();
        try {
            this.iem = getImapEntityManager();
            this.iem.removeFolder(folderId, this.sourceURI, pID);
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error accessing Mailbox", e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error removing folder", e);
        }

    }

    /**
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem updateEmail(String parentId,
                                 String mailId,
                                 String content,
                                 SyncItem syncItem)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateSyncItem FID " + parentId + " FMID " + mailId);
        }

        Email emailNew;
        try {
            emailNew = getFoundationMailFromXML(content);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        Email emailOut;
        long pID = ((Sync4jPrincipal) this.principal).getId();
        try {
            this.iem = getImapEntityManager();
            emailOut = this.iem.updateEmail(parentId, mailId, emailNew,
                                            this.sourceURI, pID);
        } catch (Exception e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new SyncSourceException(errorMessage, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest(" end update. start create SyncItem");
        }

        // create the output syncItem
        try {
            if (emailOut != null) {
                syncItem = createSyncItem(emailOut, SyncItemState.UPDATED);
            }
        } catch (Exception e) {
            throw new SyncSourceException("Error creating SyncItem: " + syncItem, e);
        }

        return syncItem;

    }

    /**
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem updateFolder(String parentId,
                                  String folderId,
                                  String content,
                                  SyncItem syncItem)

            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateFolder FID " + parentId + " and FMID " + folderId);
        }
        Folder folderNew;
        try {
            folderNew = getFoundationFolderFromXML(content);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        Folder folderOut;
        try {
            this.iem = getImapEntityManager();
            folderOut = this.iem.updateFolder(parentId, folderId, folderNew);
        } catch (Exception e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new SyncSourceException(errorMessage, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest(" end update folder. start create SyncItem");
        }

        // create the output syncItem
        try {
            if (folderOut != null) {
                syncItem = createSyncItem(folderOut, SyncItemState.UPDATED);
            }
        } catch (Exception e) {
            throw new SyncSourceException("Error creating SyncItem: " + syncItem, e);
        }

        return syncItem;

    }


}
