/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.admin;


import com.funambol.email.exception.InboxListenerConfigException;
import com.funambol.framework.config.ConfigClassLoader;
import com.funambol.framework.tools.beans.BeanFactory;
import com.funambol.framework.tools.beans.BeanInitializationException;
import com.funambol.framework.tools.beans.BeanInstantiationException;
import com.funambol.framework.tools.beans.BeanNotFoundException;
import com.funambol.server.config.ConfigTools;
import com.funambol.server.config.Configuration;

import java.net.URL;


/**
 * This class loads the parameter from the XML configuration file
 *
 * @version $Id: InboxListenerConfig.java,v 1.10 2006/08/08 09:30:10 gibi Exp $
 */
public class InboxListenerConfig {

    // --------------------------------------------------------------- CONSTANTS

    /**     */
    private final static String beanName = "email/email/InboxListener.xml";

    // -------------------------------------------------------------- PROPERTIES

    /**
     *
     */
    private String host;

    /**
     * @param host The hostname.
     */
    public void setHost(String host) {
        this.host = host;
    }

    /**
     * @return String
     */
    public String getHost() {
        return this.host;
    }

    /**
     *
     */
    private String protocol;

    /**
     * @param protocol
     */
    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }

    /**
     * @return protocol
     */
    public String getProtocol() {
        return this.protocol;
    }


    /**
     *
     */
    private boolean isSSL;

    /**
     * @param isSSL boolean
     */
    public void setIsSSL(boolean isSSL) {
        this.isSSL = isSSL;
    }

    /**
     * @return boolean
     */
    public boolean getIsSSL() {
        return this.isSSL;
    }

    /**
     *
     */
    private String keystorePath;

    /**
     * @param keystorePath
     */
    public void setKeystorePath(String keystorePath) {
        this.keystorePath = keystorePath;
    }

    /**
     * @return keystorePath
     */
    public String getKeystorePath() {
        return this.keystorePath;
    }

    /**
     * keystorePassphrase property
     */
    private String keystorePassphrase;

    /**
     * @return keystorePassphrase
     */
    public String getKeystorePassphrase() {
        return keystorePassphrase;
    }

    /**
     * @param keystorePassphrase
     */
    public void setKeystorePassphrase(String keystorePassphrase) {
        this.keystorePassphrase = keystorePassphrase;
    }


    /**
     *
     */
    private int port;

    /**
     * @param port int
     */
    public void setPort(int port) {
        this.port = port;
    }

    /**
     * @return int
     */
    public int getPort() {
        return this.port;
    }


    /**
     *
     */
    private int interval;

    /**
     * @param interval int
     */
    public void setInterval(int interval) {
        this.interval = interval;
    }

    /**
     * @return int
     */
    public int getInterval() {
        return this.interval;
    }


    /**
     *
     */
    private String listenerId;

    /**
     * @param listenerId String representing the listener ID.
     */
    public void setListenerId(String listenerId) {
        this.listenerId = listenerId;
    }

    /**
     * @return String
     */
    public String getListenerId() {
        return this.listenerId;
    }


    /**
     *
     */
    private boolean startupNotification;

    /**
     * @param startupNotification boolean
     */
    public void setStartupNotification(boolean startupNotification) {
        this.startupNotification = startupNotification;
    }

    /**
     * @return boolean
     */
    public boolean getStartupNotification() {
        return this.startupNotification;
    }


    /**
     * jdbc : url
     */
    private String jdbcUrl;

    /**
     * @param jdbcUrl String
     */
    public void setJdbcUrl(String jdbcUrl) {
        this.jdbcUrl = jdbcUrl;
    }

    /**
     * @return String
     */
    public String getJdbcUrl() {
        return this.jdbcUrl;
    }

    /**
     * jdbc : url
     */
    private String jdbcDriver; //"com.mysql.jdbc.Driver";

    /**
     * @param jdbcDriver String
     */
    public void setJdbcDriver(String jdbcDriver) {
        this.jdbcDriver = jdbcDriver;
    }

    /**
     * @return String
     */
    public String getJdbcDriver() {
        return this.jdbcDriver;
    }


    /**
     * jdbc : user
     */
    private String jdbcUser; //= "sync4j";

    /**
     * @param jdbcUser String
     */
    public void setJdbcUser(String jdbcUser) {
        this.jdbcUser = jdbcUser;
    }

    /**
     * @return String
     */
    public String getJdbcUser() {
        return this.jdbcUser;
    }


    /**
     * jdbc : password
     */
    private String jdbcPassword; //= "sync4j";

    /**
     * @param jdbcPassword String
     */
    public void setJdbcPassword(String jdbcPassword) {
        this.jdbcPassword = jdbcPassword;
    }

    /**
     * @return String
     */
    public String getJdbcPassword() {
        return this.jdbcPassword;
    }


    /**  */
    private int reloadingTime;

    /**
     * @param reloadingTime int
     */
    public void setReloadingTime(int reloadingTime) {
        this.reloadingTime = reloadingTime;
    }

    /**
     * @return String
     */
    public int getReloadingTime() {
        return this.reloadingTime;
    }


    /**
     * WS: end point
     */
    private String wsEndPoint;

    /**
     * @param wsEndPoint String
     */
    public void setWsEndPoint(String wsEndPoint) {
        this.wsEndPoint = wsEndPoint;
    }

    /**
     * @return String
     */
    public String getWsEndPoint() {
        return this.wsEndPoint;
    }

    /**
     * WS: username
     */
    private String wsUsername;

    /**
     * @param wsUsername String
     */
    public void setWsUsername(String wsUsername) {
        this.wsUsername = wsUsername;
    }

    /**
     * @return String
     */
    public String getWsUsername() {
        return this.wsUsername;
    }

    /**
     * WS: password
     */
    private String wsPassword;

    /**
     * @param wsPassword String
     */
    public void setWsPassword(String wsPassword) {
        this.wsPassword = wsPassword;
    }

    /**
     * @return String
     */
    public String getWsPassword() {
        return this.wsPassword;
    }


    /**
     * WS: SyncSource
     */
    private String wsSyncSource;

    /**
     * @param wsSyncSource String
     */
    public void setWsSyncSource(String wsSyncSource) {
        this.wsSyncSource = wsSyncSource;
    }

    /**
     * @return String
     */
    public String getWsSyncSource() {
        return this.wsSyncSource;
    }

    //------------------------------------------------------------- CONSTRUCTORS

    /**
     *
     */
    public InboxListenerConfig() {
    }

    //----------------------------------------------------------- PUBLIC METHODS


    /**
     * @return SECCConfig
     * @throws InboxListenerConfigException
     */
    public static InboxListenerConfig getConfigInstance()
            throws InboxListenerConfigException {


        ConfigClassLoader cl;
        try {
            String path = System.getProperty("inboxlistener.config");
            URL configPath = ConfigTools.fixURL(path);
            cl = new ConfigClassLoader(new URL[]{configPath},
                    Configuration.class.getClassLoader());
        } catch (Exception mue) {
            throw new InboxListenerConfigException(mue.getMessage());
        }

        InboxListenerConfig configInstance;
        try {
            Object obj = BeanFactory.getBeanInstance(cl, beanName);
            configInstance = (InboxListenerConfig) obj;
        } catch (BeanInstantiationException bie) {
            throw new InboxListenerConfigException(bie.getMessage());
        } catch (BeanInitializationException bie) {
            throw new InboxListenerConfigException(bie.getMessage());
        } catch (BeanNotFoundException bie) {
            throw new InboxListenerConfigException(bie.getMessage());
        }

        return configInstance;

    }

}
