/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.console;


import java.io.RandomAccessFile;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.DriverManager;
import java.sql.SQLException;

import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.engine.DBAccount;
import com.funambol.email.inboxlistener.engine.MailboxAccount;

/**
 * Contains methods to access to the DB tables
 *
 * @version $Id: MenuDAO.java,v 1.5 2006/08/08 09:30:17 gibi Exp $
 */
public class MenuDAO {

  // --------------------------------------------------------------- Constants

  private Logger log = Logger.getLogger(Def.LOGGER_NAME);

  /**
   *
   */
  private static final String SQL_SELECT_ACCOUNTS =
        "select username, password, listener_id from fnbl_email_account  ";


  /**
   *
   */
  private static final String SQL_INSERT_USER =
        "insert into fnbl_email_account " +
        " (username, password, listener_id, activation) values (?,?,?,?)";


  /**
   *
   */
  private static final String SQL_DELETE_USER =
        "delete from fnbl_email_account where username = ? ";

  // ------------------------------------------------------------ Private data

  /**   */
  private DBAccount dba   = null;


  // ------------------------------------------------------------- Constructor

  /**
   *
   */
  public MenuDAO(DBAccount dba) {
     this.dba = dba;
  }




  // ---------------------------------------------------------- Public Methods



  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public List getAccounts () throws InboxListenerException {

         Connection connection = null;
         PreparedStatement ps  = null;
         ResultSet rs          = null;
         List accounts         = null;

         try {

             connection = getConnection();

             ps = connection.prepareStatement(SQL_SELECT_ACCOUNTS);

             rs = ps.executeQuery();

             String  username   = null;
             String  password   = null;
             boolean isEnable   = true;
             String  listenerID = null;

             MailboxAccount ma = null;
             accounts          = new ArrayList();

             while(rs.next()) {
                 username   = rs.getString(1);
                 password   = rs.getString(2);
                 listenerID = rs.getString(3);

                 ma = new MailboxAccount(username,
                                         password,
                                         isEnable,
                                         listenerID);

                 accounts.add(ma);
             }

         } catch (SQLException e) {
             throw new InboxListenerException("Error reading DataBase (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 if (rs != null) {
                     rs.close();
                 }
                 if (ps != null) {
                     ps.close();
                 }
                 if (connection != null) {
                     connection.close();
                 }
             } catch (SQLException e){
                 log.severe("Error closing DB " + e.getMessage());
                 throw new InboxListenerException("Error closing DB (" +
                                                  e.getMessage() + ")" );
             }
         }


         return accounts;
  }


  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public void insertAccount (MailboxAccount account)
    throws InboxListenerException {

         Connection connection  = null;
         PreparedStatement ps_1 = null;
         PreparedStatement ps_2 = null;

         try {

             connection = getConnection();

             String listenerID = account.getListenerID();
             String username   = account.getUser();
             String password   = account.getPassword();
             boolean isEnable  = account.isEnable();
             String activation = null;
             if (isEnable){
                activation = "y";
             } else {
                activation = "n";
             }
             boolean alreadyIn = false;

             ps_2 = connection.prepareStatement(SQL_INSERT_USER);

             ps_2.setString(1, username);
             ps_2.setString(2, password);
             ps_2.setString(3, listenerID);
             ps_2.setString(4, activation);

             ps_2.executeUpdate();


         } catch (SQLException e) {
             throw new InboxListenerException("Error handling DataBase (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 if (ps_1 != null) {
                     ps_1.close();
                 }
                 if (ps_2 != null) {
                     ps_2.close();
                 }
                 if (connection != null) {
                     connection.close();
                 }
             } catch (SQLException e){
                 log.severe("Error closing DB " + e.getMessage());
                 throw new InboxListenerException("Error closing DB (" +
                                                  e.getMessage() + ")" );
             }
         }

  }

  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public void deleteAccount (String username)
    throws InboxListenerException {

         Connection connection  = null;
         PreparedStatement ps = null;

         try {
             connection = getConnection();
             ps = connection.prepareStatement(SQL_DELETE_USER);
             ps.setString(1, username);
             ps.executeUpdate();

         } catch (SQLException e) {
             throw new InboxListenerException("Error deleteing account (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 if (ps != null) {
                     ps.close();
                 }
                 if (connection != null) {
                     connection.close();
                 }
             } catch (SQLException e){
                 log.severe("Error closing DB " + e.getMessage());
                 throw new InboxListenerException("Error closing DB (" +
                                                  e.getMessage() + ")" );
             }
         }

  }

  // --------------------------------------------------------- Private Methods

  /**
   *
   *
   * @return Connection
   * @throws SeccException
   */
  private Connection getConnection() throws InboxListenerException {

       Connection conn = null;

       try {
         Class.forName(dba.getJdbcDriver()).newInstance();
         conn = DriverManager.getConnection(dba.getJdbcUrl(),
                                            dba.getJdbcUser(),
                                            dba.getJdbcPassword());
       } catch (ClassNotFoundException e){
         throw new InboxListenerException("Error connecting DB (Class Not Found): (" +
                                e.getMessage() + ")");
       } catch (InstantiationException e){
         throw new InboxListenerException("Error connecting DB (Instantiation Exception): (" +
                                e.getMessage() + ")");
       } catch (IllegalAccessException e){
         throw new InboxListenerException("Error connecting DB (Illegal Access): " +
                                e.getMessage() + ")");
       } catch (SQLException e){
         throw new InboxListenerException("Error connecting DB (SQLException): (" +
                                e.getMessage() + ")" );
       }

       return conn;
  }

}
