/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import java.util.List;
import java.util.ArrayList;
import java.util.logging.Logger;
import java.util.logging.Level;

import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;

import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.MailServerWrapperFactory;

import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.items.model.DefaultFolder;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.InboxListenerException;


/**
 *
 *
 * @version $Id: MailboxListener.java,v 1.12 2006/08/08 09:30:32 gibi Exp $
 */
public class MailboxListener extends Thread {

  //------------------------------------------------------------------ CONSTANTS

  private Logger log = Logger.getLogger(Def.LOGGER_NAME);

  //----------------------------------------------------------------- PROPERTIES

  /** owner of the thread */
  private List messageIDs = new ArrayList();

  /**   */
  private boolean startupNotification = false ;

  /** owner of the thread */
  private MailboxAccount mba;

  /** mail server coords */
  private MSAccount msa;

  /** webservices coords */
  private WSAccount wsa;

  /** to stop the thread  */
  private boolean go = true ;


  //--------------------------------------------------------------- CONSTRUCTORS

  /**
   *
   */
  public MailboxListener(boolean startupNotification,
                         MailboxAccount mba,
                         MSAccount msa,
                         WSAccount wsa) {
     this.startupNotification = startupNotification;
     this.mba                 = mba;
     this.msa                 = msa;
     this.wsa                 = wsa;
  }


  //------------------------------------------------------------- PUBLIC METHODS

  /**
   *
   *
   * this thread is not always connected to the mailbox.
   * every X seconds (interval) it starts a connection and read the ids
   * in the inbox folder; then compares the previous time read ids.
   * If there are new mails call the Sync4j AdminManager web service
   * in order to trigger the push message
   *
   *
   */
  public void run() {

     if (log.isLoggable(Level.FINEST)) {
        log.finest("Run MailboxListener for : " + mba.toString() +
                   " interval : " + msa.getInterval() +
                   " protocol : " + msa.getProtocol() +
                   " startupNotification : " + startupNotification );
     }

     IMailServerWrapper mswf =
             MailServerWrapperFactory.getMailServerWrapper(msa.getProtocol());

     Folder  inbox;
     boolean hasNew  = false;
     int     counter = 1;

     while (go) {

        try {
           DefaultFolder df = new DefaultFolder();
           // the IL works only the INBOX folder
           df.setInboxName(Def.FOLDER_INBOX_ENG);
           df.setInboxId(Def.FOLDER_INBOX_ID);

           mswf.openConnection(msa.getHost(),
                               "25",
                               false,
                               msa.getHost(),
                               msa.getPort()+"",
                               df,
                               mba.getUser(),
                               mba.getPassword(),
                               msa.getIsSSL(),
                               msa.getKeystorePath(),
                               msa.getKeystorePassphrase());

           inbox = mswf.getStore().getFolder(Def.FOLDER_INBOX_ENG);
           inbox.open(Folder.READ_ONLY);

           // check if there are new mails
           hasNew = hasNewMessages(inbox);

           if (log.isLoggable(Level.FINEST)) {
              log.finest("MailboxListener for : " + mba.toString() +
                         " has new mail: " + hasNew );
           }

           inbox.close(true);

           mswf.releaseConnection();


        } catch (MessagingException e) {
           e.printStackTrace();
        } catch (EmailAccessException e) {
           e.printStackTrace();
        } catch (InboxListenerException e) {
           e.printStackTrace();
        }


        if (log.isLoggable(Level.FINEST)) {
          log.finest("MailboxListener for : " + mba.toString() +
                     " call Notification Manager: " + hasNew );
        }

        // call notification system
        try {
           if (hasNew){
              if (counter == 1 && !startupNotification){
                // do nothing
              } else {
                NotificationWrapper nw = new NotificationWrapper(mba, wsa);
                nw.callNotification();
              }
           }
        } catch (Exception e){
           e.printStackTrace();
        }

        hasNew  = false;
        counter = 2;

        try{
           sleep(msa.getInterval());
        } catch (Exception e){
           e.printStackTrace();
        }


     }

  }

  /**
   *
   */
  public void stopThread(){
     if (log.isLoggable(Level.FINEST)) {
        log.finest("Shutting down MailboxListener for : " + mba.toString() );
     }
     this.go = false;
  }


  //------------------------------------------------------------ PRIVATE METHODS

  private boolean hasNewMessages(Folder inbox)
    throws InboxListenerException {

     String message_id;
     String syncLabel;
     String message_id_old;
     boolean isAlreadyInInbox;
     boolean hasNewEmail      = false;

     try {
        Message[] messages = inbox.getMessages();

        boolean loop = true;
        int     max  = messages.length;
        int     i    = 0;

        if (max > 0){

          while (loop){

            message_id       = Utility.getHeaderMessageID(messages[i]);
            syncLabel        = Utility.getHeaderSyncLabel(messages[i]);
            isAlreadyInInbox = false;

            if (syncLabel == null) {   // external mail

               if (message_id != null){

                  for (int y=0; y<messageIDs.size(); y++){
                     message_id_old = (String)messageIDs.get(y);
                     if (message_id_old.equals(message_id)){ // already in the inbox
                        isAlreadyInInbox = true;
                     }
                  }

                  if (!isAlreadyInInbox){
                    hasNewEmail = true;
                    loop = false;
                  }

               }
            }

            if (i == max-1){
              loop = false;
            } else {
              i++;
            }

          }
        }

        // replace the new messageids list
        messageIDs.clear();
        for (int y=0; y<messages.length; y++) {
           message_id = Utility.getHeaderMessageID(messages[y]);
           if (message_id != null){
              messageIDs.add(message_id);
           }
        }

     } catch (MessagingException me){
        throw new InboxListenerException("Error checking new messages", me);
     }

     return hasNewEmail;
  }

}
