/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.items.dao;


import java.util.logging.Logger;
import java.util.logging.Level;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import javax.mail.Message;
import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.MessagingException;
import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;
import com.sun.mail.imap.IMAPFolder;

import com.funambol.framework.tools.DBTools;
import com.funambol.server.tools.IdSpaceGenerator;
import com.funambol.email.items.model.ItemFolder;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.exception.EntityException;
import com.funambol.email.transport.CommonMailServerWrapper;
import com.funambol.email.util.*;
import javax.mail.internet.MimeMessage;


/**
 * This class implements methods to access data
 * in Mail Server datastore.
 *
 * @version $Id: EntityDAO.java,v 1.17 2006/08/08 09:30:47 gibi Exp $
 */
public class EntityDAO {


    // ------------------------------------------------------------ Private data

    protected Logger  log = Logger.getLogger(Def.LOGGER_NAME);

    /**     */
    protected String jndiDataSourceName = null;

    /**     */
    protected DataSource dataSource;

    /**
     * folders queries
     */
    String INSERT_FOLDER = "insert into fnbl_email_folder " +
                       "(guid, parentid, source_uri, principal, path) values " +
                       " (?,?,?,?,?)";

    String DELETE_FOLDER = "delete from fnbl_email_folder " +
                         " where guid = ? and source_uri = ? and principal = ?";

    String SELECT_FOLDER = "select guid, parentid, path from fnbl_email_folder " +
                         " where guid = ? and source_uri = ? and principal = ?";

    String SELECT_FULLPATH = "select path from fnbl_email_folder " +
                         " where guid = ? and source_uri = ? and principal = ?";

    String SELECT_FULLPATH_FROM_FID = "select path from fnbl_email_folder " +
                      " where guid like ? and source_uri = ? and principal = ?";

    String SELECT_FOLDERID = "select guid, parentid from fnbl_email_folder " +
                         " where path = ? and source_uri = ? and principal = ?";

    String SELECT_ALL_FOLDER = "select guid, parentid, path from fnbl_email_folder " +
                               " where source_uri = ? and principal = ?";

    /**
     * sent items queries
     */
    String INSERT_SENT = "insert into fnbl_email_sentpop " +
                         " (id, source_uri, principal, mail) values (?,?,?,?)";

    String UPDATE_SENT = "update fnbl_email_sentpop set id = ? " +
                         " where id = ? and source_uri = ? and principal = ?";

    String DELETE_SENT = "delete from fnbl_email_sentpop " +
                         " where id = ? and source_uri = ? and principal = ?";

    String DELETE_UNSENT = "delete from fnbl_email_cache " +
                           " where id = ? and source_uri = ? and principal = ? ";

    String SELECT_SENT = "select mail from fnbl_email_sentpop " +
                         " where id = ? and source_uri = ? and principal = ?";

    String SELECT_ALL_SENT = "select id from fnbl_email_sentpop " +
                             " where source_uri = ? and principal = ?";

    String SELECT_TWIN_SENT = "select id from fnbl_email_sentpop " +
                      " where mail like ? and source_uri = ? and principal = ?";


    // -------------------------------------------------------------- Properties


    /**
     *
     */
    public EntityDAO(){
    }

    /**
     *
     * @param jndiDataSourceName String
     */
    public EntityDAO(String jndiDataSourceName){
        this.jndiDataSourceName = jndiDataSourceName;
    }


    // -----------------------------------------------------------Public Methods

    /**
     *
     * insert a custom folder
     *
     * @param folder IMAPFolder
     * @param defFullpath String
     * @param idFolderSpace IdSpaceGenerator
     * @param source_uri String
     * @param principalId long
     * @throws EntityException
     */
    public void insertCustomFolder(IMAPFolder folder,
                                   String defFullpath,
                                   IdSpaceGenerator idFolderSpace,
                                   String source_uri,
                                   long principalId)
       throws EntityException {

        String[] allFolder = null;

        // check all subfolders and insert if nedded
        try {
            insertSubFolder(folder, idFolderSpace, source_uri, principalId);
        } catch (MessagingException me){
            throw new EntityException(me);
        }


    }


    /**
     *
     * gets the folder fullpath from the local DB
     *
     *
     * @param GUID folder id
     * @param source_uri String
     * @param principalId long
     * @return String
     * @throws EntityException
     */
    public String getFullPathFromGUID(String GUID,
                                      String source_uri,
                                      long principalId)
      throws EntityException {

        String fullpath       = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;

        try {

           lookupDataSource();
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(SELECT_FULLPATH);

           ps.setString (1, GUID        ) ;
           ps.setString (2, source_uri  ) ;
           ps.setLong   (3, principalId ) ;

           rs = ps.executeQuery();
           while(rs.next()) {
               fullpath = rs.getString(1);
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return fullpath;

    }


   /**
    *
    * gets the folder fullpath from the local DB
    * In the local db the GUID is composed by:
    * parentId + separator + objectId
    * This method look up the item with the
    * GUID = '%/FID'
    *
    *
    *
    * @param FID folder id
    * @param source_uri String
    * @param principalId long
    * @return String
    * @throws EntityException
    */
    public String getFullPathFromFID(String FID,
                                      String source_uri,
                                      long principalId)
      throws EntityException {

        String fullpath       = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;


        try {

           lookupDataSource();
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(SELECT_FULLPATH_FROM_FID);

           ps.setString (1, "%" + Def.SEPARATOR + FID ) ;
           ps.setString (2, source_uri                ) ;
           ps.setLong   (3, principalId               ) ;

           rs = ps.executeQuery();
           while(rs.next()) {
               fullpath = rs.getString(1);
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return fullpath;

    }


    /**
     *
     * gets the Folder id from the local DB
     *
     *
     * @param fullpath folder full path
     * @param source_uri String
     * @param principalId long
     * @return String
     * @throws EntityException
     */
    public String getGUIDFromFullPath(String fullpath,
                                      String source_uri,
                                      long principalId)
      throws EntityException {

        String GUID           = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;

        try {

           lookupDataSource();
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(SELECT_FOLDERID);

           ps.setString (1, fullpath    ) ;
           ps.setString (2, source_uri  ) ;
           ps.setLong   (3, principalId ) ;

           rs = ps.executeQuery();
           while(rs.next()) {
               GUID     = rs.getString(1);
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return GUID;

    }


    /**
     * This method inserts the folder in the MailServer
     *
     * @param cmsw CommonMailServerWrapper
     * @param folderName String
     * @throws EntityException
     */
    public void insertDefaultFolderInMailServer(CommonMailServerWrapper cmsw,
                                                String folderName)
      throws EntityException {

        try {
           Folder folderToInsert = cmsw.getMailDefaultFolder().getFolder(folderName);
           int type = 0; // set the right folder type
           if (!folderToInsert.exists()) {
              folderToInsert.create(Folder.HOLDS_MESSAGES);
           }
        } catch (Exception e) {
           throw new EntityException(e);
        }

    }

    /**
     *
     * serch a folder using the GUID in the local DB
     *
     *
     * @param GUID folder id
     * @param source_uri String
     * @param principalId long
     * @return ItemFolder
     * @throws EntityException
     */
    public ItemFolder getFolderFromUID(String GUID,
                                       String source_uri,
                                       long principalId)
       throws EntityException {

        ItemFolder eisFolder   = null;
        String     fullpath    = null;
        Connection connection  = null;
        PreparedStatement ps   = null;
        ResultSet rs           = null;

        try {
           lookupDataSource();
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(SELECT_FOLDER);
           ps.setString (1, GUID ) ;
           ps.setString (2, source_uri ) ;
           ps.setLong   (3, principalId ) ;
           rs = ps.executeQuery();

           String FID  = null;
           while(rs.next()) {
               GUID      = rs.getString(1);
               FID       = rs.getString(2);
               fullpath  = rs.getString(3);
           }

           if (fullpath != null) {

               String name = Utility.getFolderNameFromFullpath(fullpath);

               eisFolder = new ItemFolder(GUID, name, FID, null,
                                          null, null, null, null);
           }

           ps.close() ;
           ps = null  ;

        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return eisFolder;
    }

    /**
     *
     *
     * @param FID String
     * @param f   IMAPFolder
     * @param msg Message
     * @return EISMessage
     * @throws EntityException
     */
    public void checkMessageIDforDrafts(IMAPFolder f,
                                        CommonMailServerWrapper cmsw)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start checkMessageIDforDrafts");
        }

        try {

            Message[] messages = f.getMessages();
            int n = messages.length;

            Message msg = null;
            Message msgNew  = null;

            for (int i=0; i<n; i++) {
                msg = messages[i];

                String message_id = Utility.getHeaderMessageID(msg);

                if (message_id == null){

                    // create a new message id and set the msg
                    java.util.Random generator = new java.util.Random();
                    long random        = generator.nextLong();
                    long validity      = f.getUIDValidity();
                    long id            = f.getUID(msg);
                    String userAddress = cmsw.getUserAddress();
                    String serverOut   = cmsw.getServerOut();
                    message_id = "<" + userAddress + "." +
                                 validity    + "." +
                                 id          + "." +
                                 random      + "@" + serverOut + ">";

                    // clone the message and set the Message-ID
                    Message newMsg = new MimeMessage((MimeMessage)msg);
                    //newMsg.setHeader(Def.HEADER_LABEL_MESSAGE_ID, message_id);
                    newMsg.saveChanges();

                    // delete old
                    Message[] msgsToDelete = new Message[1];
                    msgsToDelete[0] = msg;
                    f.setFlags(msgsToDelete, new Flags(Flags.Flag.DELETED), true);

                    // append new
                    Message[] msgsToAppend = new Message[1];
                    msgsToAppend[0] = newMsg;
                    f.appendMessages(msgsToAppend);

                }

            }

        } catch (MessagingException e) {
            throw new EntityException(e);
        }

    }


  /**
   *
   *
   *
   * @param fullPath folder full path
   * @param parentId parent folder id
   * @param idFolderSpace IdSpaceGenerator
   * @param source_uri String
   * @param principalId long
   * @return String
   * @throws EntityException
   */
  public String insertFolder(String fullPath,
                             String parentId,
                             IdSpaceGenerator idFolderSpace,
                             String source_uri,
                             long principalId)
    throws EntityException {

      Connection connection = null;
      PreparedStatement  ps = null;
      String GUID           = null;
      String FID            = null;

      try {

         lookupDataSource();
         connection = dataSource.getConnection();
         ps = connection.prepareStatement(INSERT_FOLDER);

         GUID = Utility.getGUID(parentId, idFolderSpace.next());

         FID = parentId;

         ps.setString(1, GUID);
         ps.setString(2, FID);
         ps.setString(3, source_uri);
         ps.setLong(4, principalId);
         ps.setString(5, fullPath);

         ps.executeUpdate();

         ps.close() ;
         ps = null  ;

      } catch (Exception e) {
          throw new EntityException(e);
      } finally {
          DBTools.close(connection, ps, null);
      }

      return GUID;

  }


  /**
   *
   *
   * @param GUID  the Global Unique ID.
   * @param source_uri  the SyncSource URI.
   * @param principalId The principal ID.
   * @throws EntityException
   */
  public void removePopOutboxItem(String GUID,
                                  String source_uri,
                                  long principalId)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start removePopOutboxItem UID " + GUID);
        }
        Connection connection = null;
        PreparedStatement ps = null;
        try {
            lookupDataSource();
            connection = dataSource.getConnection();

            // remove from sentpop table
            ps = connection.prepareStatement(DELETE_SENT);
            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            ps.executeUpdate();
            ps.close();
            ps = null;

            // remove from cahce
            ps = connection.prepareStatement(DELETE_UNSENT);
            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            ps.executeUpdate();
            ps.close();
            ps = null;

        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

    }



  /**
   * check if the Folder is in the mail server
   *
   * @param msw - IMailServerWrapper .
   * @param fullpath - String .
   * @return true if the folder is in the Mail Server - boolean.
   * @throws EntityException
   */
  public Folder getDefaultFolder(IMailServerWrapper msw,
                                  String fullpath)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start isDefaultFolderPresent");
        }

        Folder f = null;

        try {
            f = msw.getMailDefaultFolder().getFolder(fullpath);
            if (f.exists()) {
               return f;
            }
        } catch (MessagingException e) {
            throw new EntityException(e);
        }

        return null;
    }

  /**
   *
   *
   * @param f  IMAPFolder the server outbox folder.
   * @param GUID  the Global Unique ID.
   * @param source_uri  the SyncSource URI.
   * @param principalId The principal ID.
   * @throws EntityException
   */
  public void removeImapOutboxItem(IMAPFolder f,
                                   String GUID,
                                   String source_uri,
                                   long principalId)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start removeImapOutboxItem UID " + GUID);
        }

        // remove from Mail server Outbox folder
        long FMID = Long.parseLong(Utility.getIdAfterSeparator(GUID));
        try {
            Message msgToDelete = f.getMessageByUID(FMID);
            if (msgToDelete != null) {
                Message[] msgsToDelete = new Message[1];
                msgsToDelete[0] = msgToDelete;
                f.setFlags(msgsToDelete, new Flags(Flags.Flag.DELETED), true);
            }
        } catch (MessagingException e) {
            throw new EntityException(e);
        }

        // remove from cache
        Connection connection = null;
        PreparedStatement ps = null;
        try {
            lookupDataSource();
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(DELETE_UNSENT);
            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            ps.executeUpdate();
            ps.close();
            ps = null;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

    }

  /**
    *
    *
    *
    * @param GUID String
    * @param source_uri String
    * @param principalId long
    * @throws EntityException
    */
   public void removeFolder(String GUID,
                            String source_uri,
                            long principalId)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start removeFolder UID " + GUID );
        }
        Connection connection = null;
        PreparedStatement  ps = null ;
        try {
           lookupDataSource();
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(DELETE_FOLDER);
           ps.setString (1, GUID ) ;
           ps.setString (2, source_uri ) ;
           ps.setLong   (3, principalId ) ;
           ps.executeUpdate();
           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

    }


  /**
   * This method inserts one of the default folders if needed
   *
   *
   * @param GUID String
   * @param parentId String
   * @param fullPath String
   * @param source_uri String
   * @param principalId long
   * @throws EntityException
   */
  public void insertDefaultFolderInDB(String GUID,
                                      String parentId,
                                      String fullPath,
                                      String source_uri,
                                      long principalId)
    throws EntityException {

      Connection connection = null;
      PreparedStatement  ps = null;

      try {

         lookupDataSource();
         connection = dataSource.getConnection();
         ps = connection.prepareStatement(INSERT_FOLDER);

         ps.setString (1, GUID        ) ;
         ps.setString (2, parentId    ) ;
         ps.setString (3, source_uri  ) ;
         ps.setLong   (4, principalId ) ;
         ps.setString (5, fullPath    ) ;

         ps.executeUpdate();

         ps.close() ;
         ps = null  ;

      } catch (Exception e) {
          throw new EntityException(e);
      } finally {
          DBTools.close(connection, ps, null);
      }

  }


  //-------------------------------------------------------- PROTECTED METHODS


    /**
     * Lookups the datasource
     *
     * JBOSS
     * private String jndiDataSourceName = "java:jdbc/sync4j";
     *
     * TOMCAT
     * private String jndiDataSourceName = "java:comp/env/jdbc/sync4j";
     *
     * @throws Exception if a error occurs during lookup
     */
    protected void lookupDataSource() throws Exception {

        InitialContext ctx = new InitialContext();

        try {
          // try tomcat  i.e. java:comp/env/jdbc/sync4j
          dataSource = (DataSource) ctx.lookup("java:comp/env/"+jndiDataSourceName) ;
        }
        catch (NamingException ne) {
          try {
              // try jboss i.e. java:jdbc/sync4j
              dataSource = (DataSource) ctx.lookup("java:"+jndiDataSourceName) ;
          }
          catch (Exception ex){
            try {
                // try othser j2ee server jdbc/sync4j
                dataSource = (DataSource) ctx.lookup(""+jndiDataSourceName) ;
            }
            catch (Exception e){
              throw new Exception("Data source "+ jndiDataSourceName
                                  + " not found " + e.toString());
            }
          }
        }
      }

  //------------------------------------------------------------ PRIVATE METHODS

    /**
     *
     * @param folder starting folder
     * @param fs all folders container
     * @throws MessagingException
     */
    private void insertSubFolder(IMAPFolder folder,
                                 IdSpaceGenerator idFolderSpace,
                                 String source_uri,
                                 long principalId)
       throws MessagingException, EntityException {

        javax.mail.Folder[] folderList = folder.list();

        IMAPFolder f;

        String fullpath = null;
        String GUID = null;
        String parentGUID = null;
        String parentId = null;
        String parentfullpath = null;

        for (int i = 0; i < folderList.length; i++) {

            f = ((IMAPFolder) folderList[i]);

            fullpath = f.getFullName();

            GUID = getGUIDFromFullPath(fullpath, source_uri, principalId);


            if (GUID == null){

                 parentfullpath = Utility.getParentFullpathFromFullpath(fullpath);

                 parentGUID = getGUIDFromFullPath(parentfullpath, source_uri, principalId);

                 parentId = Utility.getIdBeforeSeparator(parentGUID);

                 // insert the missed folder
                 insertFolder(fullpath, parentId,  idFolderSpace,
                              source_uri, principalId);

            }

            insertSubFolder(f, idFolderSpace, source_uri, principalId);
        }

    }

}
