/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.items.manager;


import com.funambol.email.engine.cache.CrcSyncItemInfo;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.EntityException;
import com.funambol.email.exception.SendingException;
import com.funambol.email.items.dao.ImapEntityDAO;
import com.funambol.email.items.model.DefaultFolder;
import com.funambol.email.items.model.EmailFilter;
import com.funambol.email.items.model.ItemFolder;
import com.funambol.email.items.model.ItemMessage;
import com.funambol.email.pdi.mail.Email;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.ImapMailServerWrapper;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.foundation.pdi.common.Property;
import com.funambol.server.tools.IdSpaceGenerator;
import com.sun.mail.imap.IMAPFolder;

import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.logging.Level;
import java.util.Date;
import javax.mail.Address;
import javax.mail.SendFailedException;
import javax.mail.Transport;


/**
 * @version $Id: ImapEntityManager.java,v 1.1 2006/08/08 09:31:00 gibi Exp $
 */
public class ImapEntityManager extends EntityManagerFilter {

    // -------------------------------------------------------------- Properties

    /**     */
    public ImapMailServerWrapper imsw = null;

    /**     */
    public ImapEntityDAO ied = null;

    // ------------------------------------------------------------ Costructors

    /**
     *
     */
    public ImapEntityManager() {
    }

    /**
     * @param msw_e              IMailServerWrapper
     * @param jndiDataSourceName String
     */
    public ImapEntityManager(IMailServerWrapper msw_e, String jndiDataSourceName) {
        imsw = (ImapMailServerWrapper) msw_e;
        ied = new ImapEntityDAO(jndiDataSourceName);
    }

    // ---------------------------------------------------------- Public Methods

    /**
     * deletes an email
     *
     * @param parentId    String
     * @param mailId      String
     * @param source_uri  String
     * @param principalId long
     * @throws EntityException
     */
    public void removeEmail(String parentId,
                            String mailId,
                            String source_uri,
                            long principalId)
            throws EntityException {

        IMAPFolder f = null;
        String fullpath;
        EntityException finalExc = null;

        try {

            fullpath = this.ied.getFullPathFromFID(parentId, source_uri,
                                                   principalId);

            if (log.isLoggable(Level.FINEST)) {
                log.finest("remove item in FID: " + fullpath +
                           " with FMID: " + mailId);
            }

            if (fullpath != null) {
                f = (IMAPFolder)
                        this.imsw.getMailDefaultFolder().getFolder(fullpath);
                if (f.exists()) {
                    timeStart = System.currentTimeMillis();
                    f.open(Folder.READ_WRITE);
                    this.ied.removeEmail(f, Long.parseLong(mailId));
                }
            }

        }
        catch (MessagingException me) {
            throw new EntityException(me);
        }
        catch (EntityException e) {
            throw new EntityException(e);
        }
        finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("removeItem Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                }
                catch (MessagingException me) {
                    finalExc = new EntityException(me);
                }
            }
        }
        if (finalExc != null) {
            throw finalExc;
        }

    }

    /**
     * deletes a Folder
     *
     * @param folderId    String
     * @param source_uri  String
     * @param principalId long
     * @throws EntityException
     */
    public void removeFolder(String folderId,
                             String source_uri,
                             long principalId)
            throws EntityException {

        String fullpath;
        IMAPFolder f;

        if (folderId.equalsIgnoreCase(Def.FOLDER_INBOX_ID) ||
                folderId.equalsIgnoreCase(Def.FOLDER_OUTBOX_ID) ||
                folderId.equalsIgnoreCase(Def.FOLDER_SENT_ID) ||
                folderId.equalsIgnoreCase(Def.FOLDER_DRAFTS_ID) ||
                folderId.equalsIgnoreCase(Def.FOLDER_TRASH_ID)) {
            throw new EntityException("You can't delete this Folder");
        } else {

            try {

                fullpath = this.ied.getFullPathFromGUID(folderId,
                        source_uri,
                        principalId);
                if (fullpath != null) {

                    // remove folder from MailServer
                    f = (IMAPFolder)
                            this.imsw.getMailDefaultFolder().getFolder(fullpath);

                    if (f.exists()) {
                        this.ied.removeFolderInServer(f); // the folder must be closed
                    }

                    // remove folder from Local DB
                    this.ied.removeFolder(folderId, source_uri, principalId);

                }

            } catch (MessagingException me) {
                throw new EntityException(me);
            }
        }
    }


    /**
     * updates a document defined by UID
     *
     * @param FID         String
     * @param FMID        String
     * @param emailNew    Email
     * @param source_uri  String
     * @param principalId long
     * @return Email
     * @throws EntityException
     */
    public Email updateEmail(String FID,
                             String FMID,
                             Email emailNew,
                             String source_uri,
                             long principalId)
            throws EntityException {


        ItemMessage imOut    = null;
        Email       emailOut = null;
        IMAPFolder  f        = null;
        EntityException finalExc = null;

        try {

            String fullpath = this.ied.getFullPathFromFID(FID, source_uri, principalId);

            if (log.isLoggable(Level.FINEST)) {
                log.finest("updateItem FID " + fullpath + "FID: " + FID);
            }

            if (fullpath != null) {

                f = (IMAPFolder)
                        this.imsw.getMailDefaultFolder().getFolder(fullpath);

                if (!f.exists()) {
                    return emailNew;
                }

                // the emailItem could be null
                // i.e. the client send only the read property
                // get flags
                boolean[] flags = this.setFlags(emailNew);
                boolean read      = flags[0];
                boolean forwarded = flags[1];
                boolean replied   = flags[2];

                Message msgNew = null;
                if (emailNew.getEmailItem().getStringRFC2822() != null) {
                    msgNew = createMessage(this.imsw.getSession(), emailNew, FID,
                                           false, read, forwarded, replied);
                }

                // update mail
                timeStart = System.currentTimeMillis();

                f.open(Folder.READ_WRITE);

                long uid = this.ied.updateEmail(FID, FMID, f, msgNew,
                                                read, forwarded, replied);

                Message msgOut = this.ied.getEmailFromUID(f, FID, uid);

                String messid = Utility.getHeaderMessageID(msgOut);

                String GUID = Utility.getGUID(FID, String.valueOf(uid));

                String encodingType = getEncodingType(msgOut);

                String stream = Utility.getStreamFromMessage(msgOut, encodingType);

                imOut = new ItemMessage(GUID, FID, messid, msgOut, stream,
                                        null, null, null, null, null,
                                        null, null, null, null);


            } else {
                return emailNew;
            }

        } catch (MessagingException me) {
            throw new EntityException(me);
        } catch (EntityException e) {
            throw new EntityException(e);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("updateItem Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                }
                catch (MessagingException me) {
                    finalExc = new EntityException(me);
                }
            }
        }
        if (finalExc != null) {
            throw finalExc;
        }

        // convert Message in Email
        try {
            emailOut = createFoundationMail(imOut);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("ImapEmail updateItem end");
        }

        return emailOut;
    }

    /**
     * updates a folder defined by folder id and parent id
     *
     * @param parentId  folder id String
     * @param folderId  mail id in folder String
     * @param folderNew Folder
     * @return Folder
     * @throws EntityException
     */
    public com.funambol.email.pdi.folder.Folder updateFolder(
            String parentId,
            String folderId,
            com.funambol.email.pdi.folder.Folder folderNew)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Start updateFolder FID: " + folderId);
        }

        /**
         * put the code here
         */

        if (log.isLoggable(Level.FINEST)) {
            log.finest("ImapEmail updateFolder end");
        }

        return folderNew;
    }


    /**
     * clean all folder and email except the 5 main folders
     *
     * @param allMailboxActivation boolean
     * @param source_uri           String
     * @param principalId          long
     * @throws EntityException
     */
    public void clearAllItems(boolean allMailboxActivation,
                              String source_uri,
                              long principalId)
            throws EntityException {

        IMAPFolder f = null;
        EntityException finalExc = null;

        String[] folders = {this.imsw.getDefaultFolder().getInboxName(),
                this.imsw.getDefaultFolder().getOutboxName(),
                this.imsw.getDefaultFolder().getSentName(),
                this.imsw.getDefaultFolder().getDraftName(),
                this.imsw.getDefaultFolder().getTrashName()};

        String folderName;

        // clean all emails in the main folders
        for (int y = 0; y < folders.length; y++) {
            folderName = folders[y];
            try {
                f = (IMAPFolder)
                        this.imsw.getMailDefaultFolder().getFolder(folderName);
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("ImapEmail Delete all emails in " + folderName);
                }
                if (f.exists()) {
                    timeStart = System.currentTimeMillis();
                    f.open(Folder.READ_WRITE);
                    this.ied.removeAllEmail(f);
                }
            } catch (MessagingException me) {
                throw new EntityException(me);
            } finally {
                if (f != null) {
                    try {
                        if (f.isOpen()) {
                            f.close(true);
                            timeStop = System.currentTimeMillis();
                            if (log.isLoggable(Level.FINEST)) {
                                log.finest("insertItem Execution Time: " +
                                        (timeStop - timeStart) + " ms");
                            }
                        }
                    } catch (MessagingException me) {
                        finalExc = new EntityException(me);
                    }
                }
            }
            if (finalExc != null) {
                throw finalExc;
            }
        }

        // clean all folders in the main folders
        // only if allMalboxActivation is on
        if (allMailboxActivation) {

            ArrayList fs;
            String tmpname;
            String GUID;
            IMAPFolder tmpfolder;

            for (int y = 0; y < folders.length; y++) {

                folderName = folders[y];

                try {
                    f = (IMAPFolder)
                            this.imsw.getMailDefaultFolder().getFolder(folderName);
                    if (log.isLoggable(Level.FINEST)) {
                        log.finest("ImapEmail Delete all folders in " + folderName);
                    }
                    if (f.exists()) {
                        timeStart = System.currentTimeMillis();
                        f.open(Folder.READ_WRITE);

                        fs = new ArrayList();
                        this.ied.searchSubFolderFullNames(f, fs);

                        for (int i = 0; i < fs.size(); i++) {

                            tmpname = (String) fs.get(i);

                            if (log.isLoggable(Level.FINEST)) {
                                log.finest("remove folder " + tmpname);
                            }

                            // remove folder from Mail Server
                            tmpfolder = (IMAPFolder)
                                    this.imsw.getMailDefaultFolder().getFolder(tmpname);
                            if (tmpfolder.exists()) {
                                // the folder must be closed
                                this.ied.removeFolderInServer(tmpfolder);
                            }

                            // remove folder from Local DB
                            GUID = this.ied.getGUIDFromFullPath(Def.SEPARATOR + tmpname,
                                    source_uri,
                                    principalId);

                            this.ied.removeFolder(GUID, source_uri, principalId);

                        }

                    }
                } catch (MessagingException me) {
                    throw new EntityException(me);
                } finally {
                    if (f != null) {
                        try {
                            if (f.isOpen()) {
                                f.close(true);
                                timeStop = System.currentTimeMillis();
                                if (log.isLoggable(Level.FINEST)) {
                                    log.finest("insertItem Execution Time: " +
                                            (timeStop - timeStart) + " ms");
                                }
                            }
                        } catch (MessagingException me) {
                            finalExc = new EntityException(me);
                        }
                    }
                }
                if (finalExc != null) {
                    throw finalExc;
                }
            }
        }


    }


    /**
     * @param FID         String
     * @param LUID        String
     * @param emailNew    Email
     * @param source_uri  String
     * @param principalId long
     * @return Email
     * @throws EntityException
     */
    public Email addEmail(String FID,
                          String LUID,
                          Email emailNew,
                          String source_uri,
                          long principalId)
      throws EntityException, SendingException {


        Message     msgNew   = null;
        ItemMessage imNew    = null;
        Email       emailOut = null;
        IMAPFolder  f        = null;
        String      fullpath = null;

        EntityException finalExc = null;

        try {

            if (FID.equalsIgnoreCase(Def.FOLDER_OUTBOX_ID)){

                // sending procedure (email with id starts with "O")
                FID = Def.FOLDER_SENT_ID;

                fullpath = this.ied.getFullPathFromFID(FID, source_uri, principalId);
                if (fullpath == null) {
                    return emailNew;
                }

                f = (IMAPFolder)  this.imsw.getMailDefaultFolder().getFolder(fullpath);
                if (!f.exists()) {
                    emailNew.setUID(new Property(LUID));
                    return emailNew;
                }

                // the emailItem could be null
                // i.e. the client send only the read property
                boolean[] flags = this.setFlags(emailNew);
                boolean read      = flags[0];
                boolean forwarded = flags[1];
                boolean replied   = flags[2];

                if (emailNew != null &&
                    emailNew.getEmailItem() != null &&
                    Def.ENCODE_QUOTED_PRINTABLE.equals(emailNew.getEmailItem().getEncoding())) {
                    try {
                        emailNew.getEmailItem().setPropertyValue(
                                Utility.decode(emailNew.getEmailItem().getStringRFC2822(),
                                        Def.ENCODE_QUOTED_PRINTABLE));
                        emailNew.getEmailItem().setEncoding(null);
                    }
                    catch (MessagingException e) {
                        throw new EntityException(e);
                    }
                }

                // convert Email into Message (javaMail)
                Message msg = createMessage(this.imsw.getSession(), emailNew,
                                            FID, false, read, forwarded, replied);

                String messageID = Utility.getHeaderMessageID(msg);

                sendEmail(msg);

                if (messageID != null){
                   msg.setHeader(Def.HEADER_LABEL_MESSAGE_ID,messageID);
                }

                // add in sent folder of the Mail Server
                timeStart = System.currentTimeMillis();
                f.open(Folder.READ_WRITE);
                long uid = this.ied.addEmail(FID, f, msg);

                // get the new message
                msgNew = this.ied.getEmailFromUID(f, FID, uid);

                String messid = Utility.getHeaderMessageID(msgNew);

                if (log.isLoggable(Level.FINEST)) {
                   log.finest("  FID: " + FID + " FMID:" + uid + "  Message-ID: " + messid);
                }

                // the GUID will start with "O" not "S"
                // (for mapping purpose)
                String GUID = Utility.getGUID(Def.FOLDER_OUTBOX_ID,
                                              String.valueOf(uid));

                String encodingType = getEncodingType(msgNew);

                String stream = Utility.getStreamFromMessage(msgNew, encodingType);

                imNew = new ItemMessage(GUID, Def.FOLDER_OUTBOX_ID,
                                        messid, msgNew, stream,
                                        null, null, null, null, null,
                                        null, null, null, null);

            } else {

                // adding procedure (all others emails)

                fullpath = this.ied.getFullPathFromFID(FID, source_uri, principalId);
                if (fullpath == null) {
                    return emailNew;
                }

                f = (IMAPFolder)  this.imsw.getMailDefaultFolder().getFolder(fullpath);
                if (!f.exists()) {
                    emailNew.setUID(new Property(LUID));
                    return emailNew;
                }

                // the emailItem could be null
                // i.e. the client send only the read property
                boolean[] flags = this.setFlags(emailNew);
                boolean read      = flags[0];
                boolean forwarded = flags[1];
                boolean replied   = flags[2];

                if (emailNew != null && emailNew.getEmailItem() != null &&
                        Def.ENCODE_QUOTED_PRINTABLE.equals(emailNew.getEmailItem().getEncoding())) {
                    try {
                        emailNew.getEmailItem().setPropertyValue(
                                Utility.decode(emailNew.getEmailItem().getStringRFC2822(),
                                        Def.ENCODE_QUOTED_PRINTABLE));
                        emailNew.getEmailItem().setEncoding(null);
                    }
                    catch (MessagingException e) {
                        throw new EntityException(e);
                    }
                }

                // convert Email into Message (javaMail)
                Message msg = createMessage(this.imsw.getSession(), emailNew,
                                            FID, false, read, forwarded, replied);

                // add in sent folder of the Mail Server
                timeStart = System.currentTimeMillis();

                f.open(Folder.READ_WRITE);

                long uid = this.ied.addEmail(FID, f, msg);

                // get the new message
                msgNew = this.ied.getEmailFromUID(f, FID, uid);

                String messid = Utility.getHeaderMessageID(msgNew);

                if (log.isLoggable(Level.FINEST)) {
                   log.finest("  FID: " + FID + " FMID:" + uid + "  Message-ID: " + messid);
                }

                String GUID = Utility.getGUID(FID, String.valueOf(uid));

                String encodingType = getEncodingType(msgNew);

                String stream = Utility.getStreamFromMessage(msgNew, encodingType);

                imNew = new ItemMessage(GUID, FID, messid, msgNew, stream,
                                        null, null, null, null, null,
                                        null, null, null, null);

            }

        } catch (MessagingException me) {
            throw new EntityException(me);
        } catch (SendingException e) {
            throw new SendingException(e);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("insertItem Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                }
                catch (MessagingException me) {
                    finalExc = new EntityException(me);
                }
            }
        }
        if (finalExc != null) {
            throw finalExc;
        }

        // convert ItemMessage in Email
        try {
            emailOut = createFoundationMail(imNew);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        return emailOut;

    }


    /**
     * todo understand the type of a folder (holds folders and mails)
     *
     * @param name          String
     * @param parentId      String
     * @param df            DefaultFolder
     * @param idFolderSpace IdSpaceGenerator
     * @param source_uri    String
     * @param principalId   long
     * @return Folder
     * @throws EntityException
     */
    public com.funambol.email.pdi.folder.Folder addFolder(String name,
                                                          String parentId,
                                                          DefaultFolder df,
                                                          IdSpaceGenerator idFolderSpace,
                                                          String source_uri,
                                                          long principalId)
            throws EntityException {


        com.funambol.email.pdi.folder.Folder folderOut;
        ItemFolder itemFolder = null;
        String GUID = null;
        IMAPFolder f = null; // parent folder
        String parentFullpath;
        String fullpath;
        String role;
        String creationDate;
        EntityException finalExc = null;

        try {

            parentFullpath = this.ied.getFullPathFromGUID(parentId,
                    source_uri,
                    principalId);

            fullpath = parentFullpath + Def.SEPARATOR + name;

            if (log.isLoggable(Level.FINEST)) {
                log.finest("addFolder FID:" + parentFullpath + " FMID:" + fullpath);
            }

            boolean isDefFolder = Utility.defaultFolderChecker(df, fullpath);

            if (isDefFolder) {

                GUID = Utility.getDefaultFolderId(df, fullpath);

            } else {
                f = (IMAPFolder)
                        this.imsw.getMailDefaultFolder().getFolder(parentFullpath);
                timeStart = System.currentTimeMillis();
                f.open(Folder.READ_WRITE);
                Folder folderToInsert = f.getFolder(name);
                int type = 0; // set the right folder type

                if (!folderToInsert.exists()) {
                    GUID = this.ied.addFolder(fullpath,
                            parentId,
                            folderToInsert,
                            type,
                            idFolderSpace,
                            source_uri,
                            principalId);
                }
            }

            role = Utility.getFolderRole(name, this.imsw.getDefaultFolder());

            creationDate = Utility.getFolderCreationDate();

            itemFolder = setItemFolder(GUID, name, parentId, role, creationDate);

        } catch (MessagingException me) {
            throw new EntityException("Error searching folder", me);
        } catch (EntityException ee) {
            throw new EntityException("Error creating folder", ee);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("insertFolder Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                } catch (MessagingException me) {
                    finalExc = new EntityException(me);
                }
            }
        }

        if (finalExc != null) {
            throw finalExc;
        }

        try {
            folderOut = createFoundationFolder(itemFolder);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        return folderOut;

    }


    /**
     * todo
     * 1 - get the mail id in all mailbox
     * (not only in the 5 main folders)
     * 2 - insert the filter on the default folder
     * (on the PPC the user can choose the folder to sync)
     *
     * @param filter  EmailFilter
     * @param allMailboxActivation boolean
     * @param source_uri String
     * @param principalId long
     * @return String[]
     * @throws EntityException
     */
    public String[] getAllEmails(EmailFilter filter,
                                 boolean allMailboxActivation,
                                 String source_uri,
                                 long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Imap getAllEmails() start");
        }

        // output
        String[] allitems;
        //
        String[] inboxList = null;
        String[] outboxList = null;
        String[] draftsList = null;
        String[] sentList = null;
        String[] trashList = null;
        int folderFilter = filter.getFolder();

        String fullpath;

        if (!allMailboxActivation) {
            // get only the mails in the default folders
            try {
                if (folderFilter == Def.FILTER_FOLDER_I ||
                        folderFilter == Def.FILTER_FOLDER_IO ||
                        folderFilter == Def.FILTER_FOLDER_IOS ||
                        folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    fullpath = this.imsw.getDefaultFolder().getInboxName();
                    inboxList = this.getItems(fullpath, Def.FOLDER_INBOX_ID,
                            filter, source_uri, principalId);
                }
                if (folderFilter == Def.FILTER_FOLDER_IO ||
                        folderFilter == Def.FILTER_FOLDER_IOS ||
                        folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    fullpath = this.imsw.getDefaultFolder().getOutboxName();
                    outboxList = this.getItems(fullpath, Def.FOLDER_OUTBOX_ID,
                            filter, source_uri, principalId);
                }
                if (folderFilter == Def.FILTER_FOLDER_IOS ||
                        folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    fullpath = this.imsw.getDefaultFolder().getSentName();
                    sentList = this.getItems(fullpath, Def.FOLDER_SENT_ID,
                            filter, source_uri, principalId);
                }
                if (folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    fullpath = this.imsw.getDefaultFolder().getDraftName();
                    draftsList = this.getItems(fullpath, Def.FOLDER_DRAFTS_ID,
                            filter, source_uri, principalId);
                }
                if (folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    fullpath = this.imsw.getDefaultFolder().getTrashName();
                    trashList = this.getItems(fullpath, Def.FOLDER_TRASH_ID,
                            filter, source_uri, principalId);
                }
                // put the list togheter
                allitems = Utility.mergeArray(inboxList, outboxList,
                        draftsList, sentList, trashList);
            } catch (Exception me) {
                throw new EntityException("Error getting all Emails (" +
                        me.getMessage() + ")");
            }
        } else {
            // get all the mails
            try {
                if (folderFilter == Def.FILTER_FOLDER_I ||
                        folderFilter == Def.FILTER_FOLDER_IO ||
                        folderFilter == Def.FILTER_FOLDER_IOS ||
                        folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    fullpath = this.imsw.getDefaultFolder().getInboxName();
                    inboxList = this.getItems(fullpath, Def.FOLDER_INBOX_ID,
                            filter, source_uri, principalId);
                }
                if (folderFilter == Def.FILTER_FOLDER_IO ||
                        folderFilter == Def.FILTER_FOLDER_IOS ||
                        folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    fullpath = this.imsw.getDefaultFolder().getOutboxName();
                    outboxList = this.getItems(fullpath, Def.FOLDER_OUTBOX_ID,
                            filter, source_uri, principalId);
                }
                if (folderFilter == Def.FILTER_FOLDER_IOS ||
                        folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    fullpath = this.imsw.getDefaultFolder().getSentName();
                    sentList = this.getItems(fullpath, Def.FOLDER_SENT_ID,
                            filter, source_uri, principalId);
                }
                if (folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    fullpath = this.imsw.getDefaultFolder().getDraftName();
                    draftsList = this.getItems(fullpath, Def.FOLDER_DRAFTS_ID,
                            filter, source_uri, principalId);
                }
                if (folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    fullpath = this.imsw.getDefaultFolder().getTrashName();
                    trashList = this.getItems(fullpath, Def.FOLDER_TRASH_ID,
                            filter, source_uri, principalId);
                }
                // put the list togheter
                allitems = Utility.mergeArray(inboxList, outboxList,
                        draftsList, sentList, trashList);
            } catch (Exception me) {
                throw new EntityException("Error getiing all Emails (" +
                        me.getMessage() + ")");
            }

        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Imap getAllEmails() end");
        }

        return allitems;
    }

    /**
     * if allMailboxActivation is true the method gets the
     * 5 main folders (inbox, outbox, sent, draft, trash)
     * and recursively all the subforlders
     * <p/>
     * if allMailboxActivation is false the method gets only the
     * 5 main folders (inbox, outbox, sent, draft, trash)
     *
     * @param filter               EmailFilter
     * @param allMailboxActivation boolean
     * @param source_uri           String
     * @param principalId          long
     * @return String[]
     * @throws EntityException
     */
    public String[] getAllFolders(EmailFilter filter,
                                  boolean allMailboxActivation,
                                  String source_uri,
                                  long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("imap getAllFolders() start");
        }

        String[] allfolders;
        int folderFilter = filter.getFolder();
        boolean isFolderIn = false;

        if (!allMailboxActivation) {

            // get only the default folders

            String[] inboxfolders = null;
            String[] outboxfolders = null;
            String[] sentfolders = null;
            String[] draftfolders = null;
            String[] trashfolders = null;

            if (folderFilter == Def.FILTER_FOLDER_I ||
                folderFilter == Def.FILTER_FOLDER_IO ||
                folderFilter == Def.FILTER_FOLDER_IOS ||
                folderFilter == Def.FILTER_FOLDER_IOSD ||
                folderFilter == Def.FILTER_FOLDER_IOSDT) {

                isFolderIn = this.imsw.getDefaultFolder().isInboxInMailServer();
                if (isFolderIn){
                   inboxfolders = new String[1];
                   inboxfolders[0] = Def.FOLDER_INBOX_GUID;
                }
            }
            if (folderFilter == Def.FILTER_FOLDER_IO ||
                folderFilter == Def.FILTER_FOLDER_IOS ||
                folderFilter == Def.FILTER_FOLDER_IOSD ||
                folderFilter == Def.FILTER_FOLDER_IOSDT) {

                isFolderIn = this.imsw.getDefaultFolder().isOutboxInMailServer();
                if (isFolderIn){
                   outboxfolders = new String[1];
                   outboxfolders[0] = Def.FOLDER_OUTBOX_GUID;
                }
            }
            if (folderFilter == Def.FILTER_FOLDER_IOS ||
                folderFilter == Def.FILTER_FOLDER_IOSD ||
                folderFilter == Def.FILTER_FOLDER_IOSDT) {
                isFolderIn = this.imsw.getDefaultFolder().isSentInMailServer();
                if (isFolderIn){
                   sentfolders = new String[1];
                   sentfolders[0] = Def.FOLDER_SENT_GUID;
                }
            }
            if (folderFilter == Def.FILTER_FOLDER_IOSD ||
                folderFilter == Def.FILTER_FOLDER_IOSDT) {
                isFolderIn = this.imsw.getDefaultFolder().isDraftInMailServer();
                if (isFolderIn){
                   draftfolders = new String[1];
                   draftfolders[0] = Def.FOLDER_DRAFTS_GUID;
                }
            }
            if (folderFilter == Def.FILTER_FOLDER_IOSDT) {
                trashfolders = new String[1];
                isFolderIn = this.imsw.getDefaultFolder().isTrashInMailServer();
                if (isFolderIn){
                   trashfolders[0] = Def.FOLDER_TRASH_GUID;
                }
            }

            allfolders = Utility.mergeArray(inboxfolders, outboxfolders,
                                            sentfolders, draftfolders,
                                            trashfolders);

        } else {
            // get all folders in the mailbox
            String[] inboxfolders = null;
            String[] outboxfolders = null;
            String[] sentfolders = null;
            String[] draftfolders = null;
            String[] trashfolders = null;

            String defFullpath;
            try {
                // get all subfolders in Inbox + inbox folder
                if (folderFilter == Def.FILTER_FOLDER_I ||
                    folderFilter == Def.FILTER_FOLDER_IO ||
                    folderFilter == Def.FILTER_FOLDER_IOS ||
                    folderFilter == Def.FILTER_FOLDER_IOSD ||
                    folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    isFolderIn = this.imsw.getDefaultFolder().isInboxInMailServer();
                    if (isFolderIn){
                       defFullpath = this.imsw.getDefaultFolder().getInboxName();
                       inboxfolders = this.ied.getAllFolders(filter,
                                                             this.imsw.getInboxFolder(),
                                                             defFullpath,
                                                             source_uri,
                                                             principalId);
                    }
                }
                if (folderFilter == Def.FILTER_FOLDER_IO ||
                    folderFilter == Def.FILTER_FOLDER_IOS ||
                    folderFilter == Def.FILTER_FOLDER_IOSD ||
                    folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    isFolderIn = this.imsw.getDefaultFolder().isOutboxInMailServer();
                    if (isFolderIn){
                        defFullpath = this.imsw.getDefaultFolder().getOutboxName();
                        outboxfolders = this.ied.getAllFolders(filter,
                                                               this.imsw.getOutboxFolder(),
                                                               defFullpath,
                                                               source_uri,
                                                               principalId);
                    }
                }
                if (folderFilter == Def.FILTER_FOLDER_IOS ||
                    folderFilter == Def.FILTER_FOLDER_IOSD ||
                    folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    isFolderIn = this.imsw.getDefaultFolder().isSentInMailServer();
                    if (isFolderIn){
                        defFullpath = this.imsw.getDefaultFolder().getSentName();
                        sentfolders = this.ied.getAllFolders(filter,
                                                             this.imsw.getSentFolder(),
                                                             defFullpath,
                                                             source_uri,
                                                             principalId);
                    }
                }
                if (folderFilter == Def.FILTER_FOLDER_IOSD ||
                    folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    isFolderIn = this.imsw.getDefaultFolder().isDraftInMailServer();
                    if (isFolderIn){
                       defFullpath = this.imsw.getDefaultFolder().getDraftName();
                       draftfolders = this.ied.getAllFolders(filter,
                                                             this.imsw.getDraftsFolder(),
                                                             defFullpath,
                                                             source_uri,
                                                             principalId);
                    }
                }
                if (folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    isFolderIn = this.imsw.getDefaultFolder().isTrashInMailServer();
                    if (isFolderIn){
                        defFullpath = this.imsw.getDefaultFolder().getTrashName();
                        draftfolders = this.ied.getAllFolders(filter,
                                                              this.imsw.getTrashFolder(),
                                                              defFullpath,
                                                              source_uri,
                                                              principalId);
                    }
                }

                allfolders = Utility.mergeArray(inboxfolders, outboxfolders,
                                                sentfolders, draftfolders,
                                                trashfolders);

            } catch (EmailAccessException eae) {
                throw new EntityException("Error getiing all Folders (" +
                        eae.getMessage() + ")");
            }


        }

        return allfolders;
    }


    /**
     * todo get all emails info from all folders
     * (not only from the 5 main folder)
     *
     * @param filter               EmailFilter
     * @param allMailboxActivation boolean
     * @param source_uri           String
     * @param principalId          long
     * @return CrcSyncItemInfo[]
     * @throws EntityException
     */
    public CrcSyncItemInfo[] getAllEmailsInfo(EmailFilter filter,
                                              boolean allMailboxActivation,
                                              String source_uri,
                                              long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Imap Manager getAllSyncItemInfo Start");
        }

        // all
        CrcSyncItemInfo[] syncItemInfos;
        //
        CrcSyncItemInfo[] inboxInfos = null;
        CrcSyncItemInfo[] outboxInfos = null;
        CrcSyncItemInfo[] draftsInfos = null;
        CrcSyncItemInfo[] sentInfos = null;
        CrcSyncItemInfo[] trashInfos = null;

        String path;

        if (!allMailboxActivation) {
            // get only the default folder emails info
            try {
                if (filter.getFolder() == Def.FILTER_FOLDER_I ||
                        filter.getFolder() == Def.FILTER_FOLDER_IO ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOS ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSD ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSDT) {
                    path = this.imsw.getDefaultFolder().getInboxName();
                    inboxInfos = this.getItemsInfo(path, Def.FOLDER_INBOX_ID,
                                                   filter, source_uri, principalId);
                }
                if (filter.getFolder() == Def.FILTER_FOLDER_IO ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOS ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSD ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSDT) {
                    path = this.imsw.getDefaultFolder().getOutboxName();
                    outboxInfos = this.getItemsInfo(path, Def.FOLDER_OUTBOX_ID,
                                                    filter, source_uri, principalId);
                }
                if (filter.getFolder() == Def.FILTER_FOLDER_IOS ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSD ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSDT) {
                    path = this.imsw.getDefaultFolder().getSentName();
                    sentInfos = this.getItemsInfo(path, Def.FOLDER_SENT_ID,
                                                  filter, source_uri, principalId);
                }
                if (filter.getFolder() == Def.FILTER_FOLDER_IOSD ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSDT) {
                    path = this.imsw.getDefaultFolder().getDraftName();
                    draftsInfos = this.getItemsInfo(path, Def.FOLDER_DRAFTS_ID,
                                                    filter, source_uri, principalId);
                }
                if (filter.getFolder() == Def.FILTER_FOLDER_IOSDT) {
                    path = this.imsw.getDefaultFolder().getTrashName();
                    trashInfos = this.getItemsInfo(path, Def.FOLDER_TRASH_ID,
                                                   filter, source_uri, principalId);
                }
                // put the list togheter
                syncItemInfos = Utility.mergeArray(inboxInfos, outboxInfos,
                                                   draftsInfos, sentInfos,
                                                   trashInfos);

            } catch (Exception e) {
                throw new EntityException("Error getiing all Emails Info (" +
                        e.getMessage() + ")");
            }

        } else {
            // get all emails info
            try {
                if (filter.getFolder() == Def.FILTER_FOLDER_I ||
                        filter.getFolder() == Def.FILTER_FOLDER_IO ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOS ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSD ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSDT) {
                    path = this.imsw.getDefaultFolder().getInboxName();
                    inboxInfos = this.getItemsInfo(path, Def.FOLDER_INBOX_ID,
                                                   filter, source_uri, principalId);
                }
                if (filter.getFolder() == Def.FILTER_FOLDER_IO ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOS ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSD ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSDT) {
                    path = this.imsw.getDefaultFolder().getOutboxName();
                    outboxInfos = this.getItemsInfo(path, Def.FOLDER_OUTBOX_ID,
                                                    filter, source_uri, principalId);
                }
                if (filter.getFolder() == Def.FILTER_FOLDER_IOS ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSD ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSDT) {
                    path = this.imsw.getDefaultFolder().getSentName();
                    sentInfos = this.getItemsInfo(path, Def.FOLDER_SENT_ID,
                                                  filter, source_uri, principalId);
                }
                if (filter.getFolder() == Def.FILTER_FOLDER_IOSD ||
                        filter.getFolder() == Def.FILTER_FOLDER_IOSDT) {
                    path = this.imsw.getDefaultFolder().getDraftName();
                    draftsInfos = this.getItemsInfo(path, Def.FOLDER_DRAFTS_ID,
                                                    filter, source_uri, principalId);
                }
                if (filter.getFolder() == Def.FILTER_FOLDER_IOSDT) {
                    path = this.imsw.getDefaultFolder().getTrashName();
                    trashInfos = this.getItemsInfo(path, Def.FOLDER_TRASH_ID,
                                                   filter, source_uri, principalId);
                }
                // put the list togheter
                syncItemInfos = Utility.mergeArray(inboxInfos, outboxInfos,
                                                   draftsInfos, sentInfos,
                                                   trashInfos);

            } catch (Exception e) {
                throw new EntityException("Error getiing all Emails Info (" +
                        e.getMessage() + ")");
            }
        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("Email List with " + syncItemInfos.length + " Document");
        }

        return syncItemInfos;

    }

    /**
     * @param filter               EmailFilter
     * @param allMailboxActivation boolean
     * @param source_uri           String
     * @param principalId          long
     * @return CrcSyncItemInfo[]
     * @throws EntityException
     */
    public CrcSyncItemInfo[] getAllFoldersInfo(EmailFilter filter,
                                               boolean allMailboxActivation,
                                               String source_uri,
                                               long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("ImapManager getAllFolderInfo Start");
        }

        CrcSyncItemInfo[] syncItemInfos = null;
        ItemFolder[] it;
        int folderFilter = filter.getFolder();
        boolean isFolderIn = false;

        if (!allMailboxActivation) {

            // get only the default folders
            ItemFolder[] inboxfolders = null;
            ItemFolder[] outboxfolders = null;
            ItemFolder[] sentfolders = null;
            ItemFolder[] draftfolders = null;
            ItemFolder[] trashfolders = null;

            if (folderFilter == Def.FILTER_FOLDER_I ||
                folderFilter == Def.FILTER_FOLDER_IO ||
                folderFilter == Def.FILTER_FOLDER_IOS ||
                folderFilter == Def.FILTER_FOLDER_IOSD ||
                folderFilter == Def.FILTER_FOLDER_IOSDT) {
                isFolderIn = this.imsw.getDefaultFolder().isInboxInMailServer();
                if (isFolderIn){
                    inboxfolders = new ItemFolder[1];
                    inboxfolders[0] = new ItemFolder(Def.FOLDER_INBOX_GUID,
                                                     Def.FOLDER_INBOX_ENG,
                                                     Def.FOLDER_ROOT_ID,
                                                     null, null, null, null, null);
                }
            }
            if (folderFilter == Def.FILTER_FOLDER_IO ||
                folderFilter == Def.FILTER_FOLDER_IOS ||
                folderFilter == Def.FILTER_FOLDER_IOSD ||
                folderFilter == Def.FILTER_FOLDER_IOSDT) {
                isFolderIn = this.imsw.getDefaultFolder().isOutboxInMailServer();
                if (isFolderIn){
                    outboxfolders = new ItemFolder[1];
                    outboxfolders[0] = new ItemFolder(Def.FOLDER_OUTBOX_GUID,
                                                      Def.FOLDER_OUTBOX_ENG,
                                                      Def.FOLDER_ROOT_ID,
                                                      null, null, null, null, null);
                }
            }
            if (folderFilter == Def.FILTER_FOLDER_IOS ||
                folderFilter == Def.FILTER_FOLDER_IOSD ||
                folderFilter == Def.FILTER_FOLDER_IOSDT) {
                isFolderIn = this.imsw.getDefaultFolder().isSentInMailServer();
                if (isFolderIn){
                    sentfolders = new ItemFolder[1];
                    sentfolders[0] = new ItemFolder(Def.FOLDER_SENT_GUID,
                                                    Def.FOLDER_SENT_ENG,
                                                    Def.FOLDER_ROOT_ID,
                                                    null, null, null, null, null);
                }
            }
            if (folderFilter == Def.FILTER_FOLDER_IOSD ||
                folderFilter == Def.FILTER_FOLDER_IOSDT) {
                isFolderIn = this.imsw.getDefaultFolder().isDraftInMailServer();
                if (isFolderIn){
                    draftfolders = new ItemFolder[1];
                    draftfolders[0] = new ItemFolder(Def.FOLDER_DRAFTS_GUID,
                                                     Def.FOLDER_DRAFTS_ENG,
                                                     Def.FOLDER_ROOT_ID,
                                                     null, null, null, null, null);
                }
            }
            if (folderFilter == Def.FILTER_FOLDER_IOSDT) {
                isFolderIn = this.imsw.getDefaultFolder().isTrashInMailServer();
                if (isFolderIn){
                    trashfolders = new ItemFolder[1];
                    trashfolders[0] = new ItemFolder(Def.FOLDER_TRASH_GUID,
                                                     Def.FOLDER_TRASH_ENG,
                                                     Def.FOLDER_ROOT_ID,
                                                     null, null, null, null, null);
                }
            }

            it = Utility.mergeArray(inboxfolders, outboxfolders,
                    sentfolders, draftfolders,
                    trashfolders);

        } else {

            // get all folders info in he mailbox
            ItemFolder[] inboxfolders = null;
            ItemFolder[] outboxfolders = null;
            ItemFolder[] sentfolders = null;
            ItemFolder[] draftfolders = null;
            ItemFolder[] trashfolders = null;

            ItemFolder im;

            try {

                if (folderFilter == Def.FILTER_FOLDER_I ||
                        folderFilter == Def.FILTER_FOLDER_IO ||
                        folderFilter == Def.FILTER_FOLDER_IOS ||
                        folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    isFolderIn = this.imsw.getDefaultFolder().isInboxInMailServer();
                    if (isFolderIn){
                        im = new ItemFolder(Def.FOLDER_INBOX_GUID,
                                this.imsw.getDefaultFolder().getInboxName(),
                                Def.FOLDER_ROOT_ID,
                                null, null, null, null, null);
                        inboxfolders = this.ied.getAllFoldersInfo(filter,
                                this.imsw.getInboxFolder(), im,
                                source_uri, principalId);
                    }
                }
                if (folderFilter == Def.FILTER_FOLDER_IO ||
                        folderFilter == Def.FILTER_FOLDER_IOS ||
                        folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    isFolderIn = this.imsw.getDefaultFolder().isOutboxInMailServer();
                    if (isFolderIn){
                        im = new ItemFolder(Def.FOLDER_OUTBOX_GUID,
                                this.imsw.getDefaultFolder().getOutboxName(),
                                Def.FOLDER_ROOT_ID,
                                null, null, null, null, null);
                        outboxfolders = this.ied.getAllFoldersInfo(filter,
                                this.imsw.getOutboxFolder(), im,
                                source_uri, principalId);
                    }
                }
                if (folderFilter == Def.FILTER_FOLDER_IOS ||
                        folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    isFolderIn = this.imsw.getDefaultFolder().isSentInMailServer();
                    if (isFolderIn){
                        im = new ItemFolder(Def.FOLDER_SENT_GUID,
                                this.imsw.getDefaultFolder().getSentName(),
                                Def.FOLDER_ROOT_ID,
                                null, null, null, null, null);
                        sentfolders = this.ied.getAllFoldersInfo(filter,
                                this.imsw.getSentFolder(), im,
                                source_uri, principalId);
                    }
                }
                if (folderFilter == Def.FILTER_FOLDER_IOSD ||
                        folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    isFolderIn = this.imsw.getDefaultFolder().isOutboxInMailServer();
                    if (isFolderIn){
                        im = new ItemFolder(Def.FOLDER_DRAFTS_GUID,
                                this.imsw.getDefaultFolder().getDraftName(),
                                Def.FOLDER_ROOT_ID,
                                null, null, null, null, null);
                        draftfolders = this.ied.getAllFoldersInfo(filter,
                                this.imsw.getDraftsFolder(), im,
                                source_uri, principalId);
                    }
                }
                if (folderFilter == Def.FILTER_FOLDER_IOSDT) {
                    isFolderIn = this.imsw.getDefaultFolder().isOutboxInMailServer();
                    if (isFolderIn){
                        im = new ItemFolder(Def.FOLDER_TRASH_GUID,
                                this.imsw.getDefaultFolder().getTrashName(),
                                Def.FOLDER_ROOT_ID,
                                null, null, null, null, null);
                        draftfolders = this.ied.getAllFoldersInfo(filter,
                                this.imsw.getTrashFolder(), im,
                                source_uri, principalId);
                    }
                }

                it = Utility.mergeArray(inboxfolders, outboxfolders,
                        sentfolders, draftfolders,
                        trashfolders);

            } catch (EmailAccessException eae) {
                throw new EntityException("Error getiing all Folders (" +
                        eae.getMessage() + ")");
            }

        }

        if (it != null) {
            int num = it.length;
            syncItemInfos = new CrcSyncItemInfo[num];
            CrcSyncItemInfo csii;
            for (int i = 0; i < num; i++) {
                csii = new CrcSyncItemInfo();
                csii.setKey((it[i]).getGUID());
                csii.setLastCrc(1);
                syncItemInfos[i] = csii;
            }
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Folder List with " + syncItemInfos.length + " Document");
            }
        }

        return syncItemInfos;

    }

    /**
     *
     * checks if a mail match the filter
     * this method gets all email in the folder considering the filter
     * verifies if the key is in the message list
     *
     * @param filter  EmailFilter
     * @param GUID String
     * @param parentId String
     * @param mailId String
     * @param source_uri  String
     * @param principalId long
     * @return true if the mail match the filter clause
     * @throws EntityException
     */
    public boolean isEmailInFilter(EmailFilter filter,
                                   String GUID,
                                   String parentId,
                                   String mailId,
                                   String source_uri,
                                   long principalId)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("isEmailInFilter in folder " + parentId + " mail: " + mailId);
        }

        boolean    isInFilter    = false;
        IMAPFolder f             = null;
        EntityException finalExc = null;
        ArrayList  keys          = new ArrayList();

        // get all email in the path using the filter clause
        try {

            long FMID    = Long.parseLong(mailId);

            String fullpath = this.ied.getFullPathFromFID(parentId, source_uri,
                                                          principalId);
            f = (IMAPFolder)
                    this.imsw.getMailDefaultFolder().getFolder(fullpath);

            if (!f.exists()) {
                return false;
            }

            timeStart = System.currentTimeMillis();

            f.open(Folder.READ_WRITE);

            Message[] messages = this.ied.getAllEmails(f, filter);

            if (messages != null) {
                String keytemp = null;
                for (int i = 0; i < messages.length; i++) {
                    keytemp = String.valueOf(f.getUID(messages[i]));
                    if (keytemp.equals(mailId)){
                       isInFilter = true;
                       break;
                    }
                }
            }

        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("getItemByUID Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                } catch (MessagingException me) {
                    finalExc = new EntityException(me);
                }
            }
        }
        if (finalExc != null) {
            throw finalExc;
        }

        return isInFilter;

    }


    /**
     *
     * checks if a mail match the filter
     *
     * @param filter EmailFilter
     * @param parentId String
     * @param GUID String
     * @param email  Email
     * @return true if the message match the filter
     * @throws EntityException
     */
    public boolean isEmailInFilter(EmailFilter filter,
                                   String parentId,
                                   String GUID,
                                   Email email)
      throws EntityException {

        boolean isInFilter = false;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("isEmailInFilter in folder " + parentId + " GUID: " + GUID);
        }

        try {

            // get Properties
            boolean[] flags = this.setFlags(email);
            boolean read      = flags[0];
            boolean forwarded = flags[1];
            boolean replied   = flags[2];

            Message msg = createMessage(this.imsw.getSession(), email, parentId,
                                        false,  read,  forwarded, replied);

            // check if this message is included in the filter

            if (filter.getId() != null) {
                // check GUID
                if (filter.getId().equals(GUID)){
                    isInFilter = true;
                } else {
                    isInFilter = false;
                }
            } else {
                // check date
                Date msgdate = Utility.getHeaderDate(msg, this.imsw.getLocale());
                if ((filter.getTime()!=null) && (msgdate!=null)){
                    if (hasMatchedDate(msgdate,filter)){
                        isInFilter = true;
                    } else {
                        isInFilter = false;
                    }
                } else {
                    // check filter size
                    if (hasMatchedSize(msg,filter)){
                        isInFilter = true;
                    } else {
                        isInFilter = false;
                    }
                }
            }

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return isInFilter;
    }

    /**
     *
     * checks if a mail match the filter
     *
     * @param filter EmailFilter
     * @param parentId String
     * @param GUID String
     * @param email  Email
     * @return true if the message match the filter
     * @throws EntityException
     */
    /*
    public boolean isEmailInFilter(EmailFilter filter,
                                   String parentId,
                                   String GUID,
                                   Email email)
      throws EntityException {

        boolean isInFilter = false;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("isEmailInFilter in folder " + parentId + " GUID: " + GUID);
        }

        try {

            // get Properties
            boolean[] flags = this.setFlags(email);
            boolean read      = flags[0];
            boolean forwarded = flags[1];
            boolean replied   = flags[2];

            Message msg = createMessage(this.imsw.getSession(), email, parentId,
                                        false,  read,  forwarded, replied);

            // check if this message is included in the filter

            if (filter.getId() != null) {
                // check GUID
                if (filter.getId().equals(GUID)){
                    isInFilter = true;
                } else {
                    isInFilter = false;
                }
            } else {
                // check date
                Date msgdate = Utility.getHeaderDate(msg, this.imsw.getLocale());
                if ((filter.getTime()!=null) && (msgdate!=null)){
                    if (hasMatchedDate(msgdate,filter)){
                        isInFilter = true;
                    } else {
                        isInFilter = false;
                    }
                } else {
                    // check filter size
                    if (hasMatchedSize(msg,filter)){
                        isInFilter = true;
                    } else {
                        isInFilter = false;
                    }
                }
            }

        } catch (Exception e) {
                throw new EntityException(e);
        }

        return isInFilter;
    }
    */

    /**
     * @param filter EmailFilter
     * @param mailId  mail id; String
     * @param parentId parent id; String
     * @param status status of the item; char
     * @param source_uri  String
     * @param pID princiapal ID long
     * @return Email
     * @throws EntityException
     */
    public Email getEmailFromUID(EmailFilter filter,
                                 String mailId,
                                 String parentId,
                                 char status,
                                 String source_uri,
                                 long pID)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByUID in folder " + parentId +
                       " mail: " + mailId + " status " + status );
        }

        long FMID = Long.parseLong(mailId);

        String fullpath = this.ied.getFullPathFromFID(parentId, source_uri, pID);

        ItemMessage im  = getItem(fullpath, parentId, FMID, filter, status,
                                  source_uri, pID);

        Email emailOut = null;
        if (im != null) {
            try {
                emailOut = createFoundationMail(im);
            } catch (Exception e) {
                throw new EntityException(e);
            }
        }

        return emailOut;
    }

    /**
     * get folder from GUID
     *
     * @param filter      EmailFilter
     * @param GUID        folder id (parentid + / + folderid)
     * @param source_uri  String
     * @param principalId long
     * @return Folder
     * @throws EntityException
     */
    public com.funambol.email.pdi.folder.Folder getFolderFromUID(EmailFilter filter,
                                                                 String GUID,
                                                                 String source_uri,
                                                                 long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getFolderFromUID start " + GUID);
        }

        com.funambol.email.pdi.folder.Folder folderOut = null;
        ItemFolder im;
        IMAPFolder f;
        String fullpath;
        String name;
        String parentFullpath;
        String FID;
        String role;
        String creationDate;

        try {

            fullpath = this.ied.getFullPathFromGUID(GUID,
                    source_uri,
                    principalId);
            if (fullpath != null) {
                f = (IMAPFolder)
                        this.imsw.getMailDefaultFolder().getFolder(fullpath);
                // if folder doesn't exists
                if (!f.exists()) {
                    return folderOut;
                }
                // if folder exists
                name = f.getName();
                parentFullpath = f.getParent().getFullName();
                FID = this.ied.getGUIDFromFullPath(parentFullpath,
                        source_uri,
                        principalId);

                role = Utility.getFolderRole(name, this.imsw.getDefaultFolder());

                creationDate = Utility.getFolderCreationDate();

                im = setItemFolder(GUID, name, FID, role, creationDate);

            } else {
                return folderOut;
            }

        } catch (Exception e) {
            throw new EntityException(e);
        }

        // convert ServerFolder in Folder
        if (im != null) {
            try {
                folderOut = createFoundationFolder(im);
            } catch (Exception e) {
                throw new EntityException(e);
            }
        }

        return folderOut;

    }

    /**
     * search the twin of the folder "name".
     * if the folder already exists the method returns the GUID
     *
     * @param name        String
     * @param parentId    String
     * @param source_uri  String
     * @param principalId long
     * @return com.funambol.email.pdi.folder.Folder
     * @throws EntityException
     */
    public String getFolderFromName(String name,
                                    String parentId,
                                    String source_uri,
                                    long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getFolderFromName start Name: " + name +
                    " parent ID: " + parentId);
        }

        IMAPFolder f;
        String parentFullpath;
        String fullpath;
        String GUID = null;

        /**
         * we should add some control.
         * We can use the device information
         */
        String nametmp = Utility.convertNokiaToDefault(name,
                this.imsw.getDefaultFolder());
        // it is a nokia folder
        if (nametmp != null) {
            name = nametmp;
        }

        try {

            if (parentId.equalsIgnoreCase(Def.FOLDER_ROOT_ID)) {
                parentFullpath = Def.FOLDER_ROOT_ENG;
            } else {
                parentFullpath = this.ied.getFullPathFromGUID(parentId,
                        source_uri,
                        principalId);
            }

            fullpath = parentFullpath + Def.SEPARATOR + name;

            if (fullpath.startsWith("/")) {
                fullpath = fullpath.substring(1, fullpath.length());
            }

            // check in the Mail Server
            f = (IMAPFolder) this.imsw.getMailDefaultFolder().getFolder(fullpath);

            if (!f.exists()) {
                return GUID;
            } else {
                // if folder exists get GUID from DB
                // if GUID is null there is inconsistent data in the DB
                GUID = this.ied.getGUIDFromFullPath(fullpath,
                        source_uri,
                        principalId);
            }

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return GUID;

    }


    /**
     * checks if a mail is already inserted in the db
     * and returns the Document in the Mail Server
     * we use header MESSAGE-ID as search criteria
     *
     * @param FID         String
     * @param FMID        String
     * @param ctmp        Email
     * @param source_uri  String
     * @param principalId long
     * @return Email
     * @throws EntityException
     */
    public String getEmailFromClause(String FID,
                                     String FMID,
                                     Email ctmp,
                                     String source_uri,
                                     long principalId)
            throws EntityException {

        String messageIdExt;
        String GUID              = null;
        IMAPFolder f             = null;
        EntityException finalExc = null;

        String fullpath = this.ied.getFullPathFromFID(FID, source_uri, principalId);

        if (fullpath == null) {
            return GUID;
        } else {

            try {

                f = (IMAPFolder)
                        this.imsw.getMailDefaultFolder().getFolder(fullpath);

                // check folder
                if (!f.exists()) {
                    return GUID;
                }

                // get Properties
                boolean[] flags = this.setFlags(ctmp);
                boolean read      = flags[0];
                boolean forwarded = flags[1];
                boolean replied   = flags[2];

                Message msgExt = createMessage(this.imsw.getSession(),
                                               ctmp, FID, false,
                                               read, forwarded, replied);

                messageIdExt = Utility.getHeaderMessageID(msgExt);

                timeStart = System.currentTimeMillis();

                f.open(Folder.READ_WRITE);

                if (messageIdExt != null) {
                    GUID = this.ied.getEmailFromMessageID(f, FID, messageIdExt);
                }

            } catch (Exception e) {
                throw new EntityException(e);
            } finally {
                if (f != null) {
                    try {
                        if (f.isOpen()) {
                            f.close(true);
                            timeStop = System.currentTimeMillis();
                            if (log.isLoggable(Level.FINEST)) {
                                log.finest("getItemByWhereClause Execution Time: " +
                                        (timeStop - timeStart) + " ms");
                            }
                        }
                    } catch (MessagingException me) {
                        finalExc = new EntityException(me);
                    }
                }
            }

            if (finalExc != null) {
                throw finalExc;
            }
        }

        return GUID;

    }



    //---------------------------------------------------------- PRIVATE METHODS



    /**
     * @param fullpath    parent folder
     * @param FMID        mail id (in the folder, not unique)
     * @param filter      EmailFilter
     * @param status      status of the Item; char
     * @param source_uri  String
     * @param principalId long
     * @return ItemMessage
     * @throws EntityException
     */
    private ItemMessage getItem(String fullpath,
                                String FID,
                                long FMID,
                                EmailFilter filter,
                                char status,
                                String source_uri,
                                long principalId)
            throws EntityException {

        ItemMessage im = null;
        IMAPFolder f = null;
        EntityException finalExc = null;

        try {

            f = (IMAPFolder) this.imsw.getMailDefaultFolder().getFolder(fullpath);

            if (!f.exists()) {
                return im;
            }

            timeStart = System.currentTimeMillis();

            f.open(Folder.READ_WRITE);

            Message msg = this.ied.getEmailFromUID(f, FID, FMID);

            im = getPart(f, String.valueOf(FMID), filter, msg,
                         status, source_uri, principalId);

        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("getItemByUID Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                } catch (MessagingException me) {
                    finalExc = new EntityException(me);
                }
            }
        }
        if (finalExc != null) {
            throw finalExc;
        }
        return im;
    }


    /**
     * get messages from folders
     *
     * @param fullpath String
     * @param folderId String
     * @param filter   EmailFilter
     * @return String[]
     * @throws EntityException
     */
    private String[] getItems(String fullpath,
                              String folderId,
                              EmailFilter filter,
                              String source_uri,
                              long principalId)
      throws EntityException {

        if (log.isLoggable(Level.INFO)) {
            log.info("ImapEntityManager.getItems for source_uri: " + source_uri +
                    " and principalId: " + principalId + ".");
        }


        String[] msgIds = null;

        IMAPFolder f = null;
        EntityException finalExc = null;

        try {

            f = (IMAPFolder)
                    this.imsw.getMailDefaultFolder().getFolder(fullpath);

            // check folder
            if (!f.exists()) {
                return msgIds;
            }

            timeStart = System.currentTimeMillis();

            f.open(Folder.READ_WRITE);

            Message[] messages = this.ied.getAllEmails(f, filter);


            String FMID;

            if (messages != null) {

                int numMsgs = messages.length;
                msgIds = new String[numMsgs];

                for (int i = 0; i < numMsgs; i++) {
                    FMID = String.valueOf(f.getUID(messages[i]));
                    msgIds[i] = Utility.getGUID(folderId, FMID);
                }

            }

        } catch (MessagingException e) {
            throw new EntityException(e);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("getItems Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                } catch (MessagingException me) {
                    finalExc = new EntityException(me);
                }
            }
        }
        if (finalExc != null) {
            throw finalExc;
        }

        return msgIds;

    }

    /**
     * get messages from folders
     *
     * @param fullpath path of the folder
     * @param folderId parent id
     * @param filter   EmailFilter
     * @return CrcSyncItemInfo[]
     * @throws EntityException
     */
    private CrcSyncItemInfo[] getItemsInfo(String fullpath,
                                           String folderId,
                                           EmailFilter filter,
                                           String source_uri,
                                           long principalId)
            throws EntityException {


        if (log.isLoggable(Level.FINEST)) {
            log.finest("Imap Manager getItemsInfo start for source_uri: " + source_uri +
                    " and principalId: " + principalId + ".");
        }

        List allSyncItemInfo = new ArrayList();
        CrcSyncItemInfo syncItemInfo;
        CrcSyncItemInfo[] syncItemInfos = null;
        Message msg;
        long lastCrc;
        int itemsNum = 0;
        IMAPFolder f = null;
        EntityException finalExc = null;

        try {

            f = (IMAPFolder)
                    this.imsw.getMailDefaultFolder().getFolder(fullpath);

            // check folder
            if (!f.exists()) {
                return syncItemInfos;
            }


            timeStart = System.currentTimeMillis();

            f.open(Folder.READ_WRITE);

            Message[] items = this.ied.getAllEmails(f, filter);

            if (items != null) {
                itemsNum = items.length;
            }


            String GUID;

            for (int i = 0; i < itemsNum; i++) {

                msg = items[i];

                lastCrc = Utility.setLastCrc(msg);

                GUID = Utility.getGUID(folderId, String.valueOf(f.getUID(msg)));

                syncItemInfo = new CrcSyncItemInfo();

                syncItemInfo.setLastCrc(lastCrc);
                syncItemInfo.setKey(GUID);

                allSyncItemInfo.add(syncItemInfo);

            }

        } catch (MessagingException e) {
            throw new EntityException(e);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("getItemsInfo Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                } catch (MessagingException me) {
                    finalExc = new EntityException(me);
                }
            }
        }

        if (finalExc != null) {
            throw finalExc;
        }

        syncItemInfos = (CrcSyncItemInfo[]) allSyncItemInfo.toArray(new CrcSyncItemInfo[0]);

        return syncItemInfos;

    }

    /**
     *
     * @param f IMAPFolder
     * @param FMID mail id in the folder
     * @param filter EmailFilter
     * @param msg Message
     * @param status char
     * @param source_uri String
     * @param principalId long
     * @return ItemMessage
     * @throws EntityException
     */
    private ItemMessage getPart(IMAPFolder f,
                                String FMID,
                                EmailFilter filter,
                                Message msg,
                                char status,
                                String source_uri,
                                long principalId)
            throws EntityException {

        ItemMessage im = null;

        if (msg != null) {

            try {

                String parentfullpath = f.getFullName();
                if (parentfullpath.equals("INBOX")) {
                    parentfullpath = Def.FOLDER_INBOX_ENG;
                }

                String FID = this.ied.getGUIDFromFullPath(parentfullpath,
                                                          source_uri,
                                                          principalId);

                Locale loc = this.imsw.getLocale();

                Session session = this.imsw.getSession();

                if (filter.getSize() == Def.FILTER_SIZE_H) {
                    im = get_H(session, FID, FMID, msg, status, loc);
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B) {
                    im = get_HB(session, FID, FMID, msg, status, loc, -1);
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B_PERC) {
                    im = get_HB(session, FID, FMID, msg, status, loc, filter.getNumBytes());
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B_A) {
                    im = get_HBA(session, FID, FMID, msg, status, loc, -1);
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B_A_PERC) {
                    im = get_HBA(session, FID, FMID, msg, status, loc, filter.getNumBytes());
                }

            } catch (EntityException me) {
                throw new EntityException(me);
            }

        }

        return im;
    }


}
