/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.transport  ;

import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.Locale;
import java.util.Properties;

import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Folder;
import com.sun.net.ssl.internal.ssl.Provider;

import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.util.Def;
import com.funambol.email.items.model.DefaultFolder;


/**
 *
 * <p>
 * Create a connection with a Mail Server
 * </p>
 *
 * @version $Id: CommonMailServerWrapper.java,v 1.12 2006/08/08 09:32:46 gibi Exp $
 */
public abstract class CommonMailServerWrapper
        implements IMailServerWrapper
{

    // --------------------------------------------------------------- Constants

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    protected String RESOURCE_NAME  = "com/funambol/email/resources/Lang";


    // -------------------------------------------------------------- Properties

    /**     */
    protected Session session;
    /**     */
    protected Store store ;
    /**     */
    protected DefaultFolder defaultFolder ;
    /**     */
    protected Folder mailDefaultFolder ;
    /**     */
    protected Locale locale;
    /**     */
    protected String serverOut;
    /**     */
    protected String userAddress;


    /**
     *
     *
     * @param outgoingServer String
     * @param outgoingPort String
     * @param incomingServer String
     * @param incomingPort String
     * @param defaultFolder DefaultFolder
     * @param user String
     * @param password String
     * @param isSSL boolean
     * @throws EmailAccessException
     */
    public abstract void openConnection(String  outgoingServer,
                                        String  outgoingPort,
                                        boolean smtpAuth,
                                        String  incomingServer,
                                        String  incomingPort,
                                        DefaultFolder defaultFolder,
                                        String  user,
                                        String  password,
                                        boolean isSSL,
                                        String keystorePath,
                                        String keystorePassphrase)
      throws EmailAccessException ;

    /**
     *
     * @throws EmailAccessException
     */
    public abstract void releaseConnection()  throws EmailAccessException ;

    /**
     *
     *
     *
     * @param server String
     * @param incomingPort String
     * @param user String
     * @param password String
     * @param isSSL boolean
     * @throws EmailAccessException
     */
    public void checkUser(String server,
                          String incomingPort,
                          String user,
                          String password,
                          boolean isSSL,
                          String keystorePath,
                          String keystorePassphrase)
     throws EmailAccessException{

      if (log.isLoggable(Level.FINEST)) {
          log.finest("\nCONNECTION PARAMs"+
                     "\nserver       " + server+
                     "\nport         " + incomingPort+
                     "\nprotocol     " + this.getClass().getName()+
                     "\nuser         " + user+
                     "\nSSL          " + isSSL+
                     "\nKeystorePath " + keystorePath);
      }

      // contains for properties
      Properties props = new Properties();

      // provider for a SSL connection
      Provider provider = null;

      if (isSSL) {
        setConnectionSSL(props, server, "25", false, server, incomingPort,
                         provider, keystorePath, keystorePassphrase);
      } else {
        setConnectionNormal(props, server, "25", false, server, incomingPort);
      }

      boolean isOpen = multipleOpenConnection(props, server, user, password);

      /*
      boolean isOpen = false;
      String error = "";
      int islock   = -1;

      try{
          this.session = Session.getInstance(props);
          this.store   = session.getStore();
          this.store.connect(server, user, password);
          isOpen = true;
      } catch (Exception e1) {
          error = e1.getMessage();
          islock = error.indexOf("lock");
          if (islock != -1){
              try {
                 Thread.sleep(3000);
                 this.session = Session.getInstance(props);
                 this.store   = session.getStore();
                 this.store.connect(server, user, password);
                 isOpen = true;
              } catch (Exception e2) {
                  error = e1.getMessage();
                  islock = error.indexOf("lock");
                  if (islock != -1){
                      try {
                         Thread.sleep(3000);
                         this.session = Session.getInstance(props);
                         this.store   = session.getStore();
                         this.store.connect(server, user, password);
                         isOpen = true;
                      } catch (Exception e3) {
                         log.severe("Error Opening Connection Store: " + e3);
                         log.throwing(this.getClass().getName(), "openConnection", e3);
                         throw new EmailAccessException("Error Opening Connection Store (" + e3.getMessage() + ")");
                      }
                  } else {
                     log.severe("Error Opening Connection Store: " + e2);
                     log.throwing(this.getClass().getName(), "openConnection", e2);
                     throw new EmailAccessException("Error Opening Connection Store (" + e2.getMessage() + ")");
                  }
              }
          } else {
              log.severe("Error Opening Connection Store: " + e1);
              log.throwing(this.getClass().getName(), "openConnection", e1);
              throw new EmailAccessException("Error Opening Connection Store (" + e1.getMessage() + ")");
          }
      }
      */

      if (isOpen){
          try {
             this.store.close();
             this.store = null;
             this.session = null;
             props.clear();
          } catch (Exception e) {
                throw new EmailAccessException("Error closing store " + e.toString());
          }
          log.info("Check User operation done");
      } else {
          log.info("Error Checking User - Synchronization Interrupt");
      }

    }

    /**
     *
     * @param props Properties
     * @param outgoingServer String
     * @param outgoingPort String
     * @param incomingServer String
     * @param incomingPort String
     */
    protected abstract void setConnectionNormal(Properties props,
                                                String outgoingServer,
                                                String outgoingPort,
                                                boolean smtpAuth,
                                                String incomingServer,
                                                String incomingPort);

    /**
     *
     * @param props Properties
     * @param outgoingServer String
     * @param outgoingPort String
     * @param incomingServer String
     * @param incomingPort String
     * @param provider Provider
     */
    protected abstract void  setConnectionSSL(Properties props,
                                              String outgoingServer,
                                              String outgoingPort,
                                              boolean smtpAuth,
                                              String incomingServer,
                                              String incomingPort,
                                              Provider provider,
                                              String keystorePath,
                                              String kestorePassphrase);
    /**
     *
     *
     * @return Session
     */
    public Session getSession(){
       return this.session;
    }

    /**
     *
     * @return Store
     */
    public Store getStore(){
       return this.store;
    }

    /**
     *
     * @return Folder
     */
    public Folder getMailDefaultFolder(){
       return this.mailDefaultFolder;
    }

    /**
     *
     * @return DefaultFolder
     */
    public DefaultFolder getDefaultFolder(){
       return this.defaultFolder;
    }


    /**
     *
     *
     * @return Locale
     */
    public Locale getLocale(){
       return this.locale;
    }

    /**
     * used in the creation of the Message-ID
     * see IMAP-based  getEmailFromUID
     *
     * @return serverOut
     */
    public String getServerOut(){
       return this.serverOut;
    }

    /**
     * used in the creation of the Message-ID
     * see IMAP-based  getEmailFromUID
     *
     * @return String
     */
    public String getUserAddress(){
       return this.userAddress;
    }

    // --------------------------------------------------------- PRIVATE METHODS

    /**
     *
     *
     */
    private boolean multipleOpenConnection(Properties props,
                                           String server,
                                           String user,
                                           String password)
     throws EmailAccessException {

      boolean isOpen = false;
      String error = "";
      int islock   = -1;

      try {

          this.session = Session.getInstance(props);
          this.store   = session.getStore();
          this.store.connect(server, user, password);
          isOpen = true;

      } catch (Exception e1) {
          error = e1.getMessage();
          islock = error.indexOf("lock");
          if (islock != -1){
              try {
                 Thread.sleep(3000);
                 this.session = Session.getInstance(props);
                 this.store   = session.getStore();
                 this.store.connect(server, user, password);
                 isOpen = true;
              } catch (Exception e2) {
                  error = e1.getMessage();
                  islock = error.indexOf("lock");
                  if (islock != -1){
                      try {
                         Thread.sleep(3000);
                         this.session = Session.getInstance(props);
                         this.store   = session.getStore();
                         this.store.connect(server, user, password);
                         isOpen = true;
                      } catch (Exception e3) {
                         log.severe("Error Opening Connection Store: " + e3);
                         log.throwing(this.getClass().getName(), "openConnection", e3);
                         throw new EmailAccessException("Error Opening Connection Store (" + e3.getMessage() + ")");
                      }
                  } else {
                     log.severe("Error Opening Connection Store: " + e2);
                     log.throwing(this.getClass().getName(), "openConnection", e2);
                     throw new EmailAccessException("Error Opening Connection Store (" + e2.getMessage() + ")");
                  }
              }
          } else {
              log.severe("Error Opening Connection Store: " + e1);
              log.throwing(this.getClass().getName(), "openConnection", e1);
              throw new EmailAccessException("Error Opening Connection Store (" + e1.getMessage() + ")");
          }
      }

      return isOpen;

    }

}
