/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.transport  ;

import java.util.logging.Level;
import java.util.Properties;
import java.security.Security;
import javax.mail.Session;
import com.sun.net.ssl.internal.ssl.Provider;
import com.sun.mail.imap.IMAPFolder;
import javax.mail.MessagingException;

import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.items.model.DefaultFolder;


/**
 *
 * <p>
 * Create a connection with a Mail Server
 * using the IMAP protocol
 * </p>
 *
 * @version $Id: ImapMailServerWrapper.java,v 1.14 2006/08/08 09:32:46 gibi Exp $
 */
public class ImapMailServerWrapper
        extends CommonMailServerWrapper
        implements IMailServerWrapper
{

    // -------------------------------------------------------------- Properties


    // ---------------------------------------------------------- PUBLIC METHODS


    /**
     * opens the connection with the mail server
     *
     *
     * @param outgoingServer SMPT address i.e. 192.168.0.1
     * @param outgoingPort 25 def.
     * @param incomingServer IMAP address i.e. 192.168.0.1
     * @param incomingPort 143 / 993 IMAP def.
     * @param df inbox, outbox, sent items,...
     * @param user String
     * @param password String
     * @param isSSL true/false
     * @throws EmailAccessException
     */
    public void openConnection(String  outgoingServer,
                               String  outgoingPort,
                               boolean smtpAuth,
                               String  incomingServer,
                               String  incomingPort,
                               DefaultFolder df,
                               String  user,
                               String  password,
                               boolean isSSL,
                               String keystorePath,
                               String keystorePassphrase)
      throws EmailAccessException {

      if (log.isLoggable(Level.FINEST)) {
          log.finest("\nCONNECTION PARAMs"+
                     "\nout          " + outgoingServer+
                     "\nport         " + outgoingPort+
                     "\nsmtpauth     " + smtpAuth+
                     "\nin           " + incomingServer+
                     "\nport         " + incomingPort+
                     "\nprotocol     " + this.getClass().getName()+
                     "\nuser         " + user+
                     "\nSSL          " + isSSL+
                     "\nkeystore     " + keystorePath+
                     "\npassphrase   " + keystorePassphrase);
      }

      // set info
      this.userAddress = user;
      this.serverOut   = outgoingServer;

      // containe for properties
      Properties props = new Properties();

      // provider for a SSL connection
      Provider provider = null;

      if (isSSL) {
        this.setConnectionSSL(props, outgoingServer, outgoingPort,
                              smtpAuth, incomingServer, incomingPort,
                              provider, keystorePath, keystorePassphrase);
      } else {
        this.setConnectionNormal(props, outgoingServer, outgoingPort,
                                 smtpAuth, incomingServer, incomingPort);
      }

      try{
         if (smtpAuth){
            ConnectorAuthenticator auth = new ConnectorAuthenticator();
            auth.setUserName(user);
            auth.setPassword(password);
            this.session = Session.getInstance(props, auth);
         } else {
            this.session = Session.getInstance(props, null);
         }
      } catch (Exception e) {
         log.severe("Error Setting Authentication for Server: " + e);
         log.throwing(this.getClass().getName(), "openConnection", e);
	 throw new EmailAccessException("Error Setting Authentication for Server (" + e.getMessage() + ")");
      }

      boolean isOpen = multipleOpenConnection(incomingServer, user, password);
      /*
      boolean isOpen = false;
      try{
         this.store = session.getStore();
         this.store.connect(incomingServer, user, password);
         isOpen = true;
      } catch (Exception e1) {
          try{
             Thread.sleep(3000);
             this.store = session.getStore();
             this.store.connect(incomingServer, user, password);
             isOpen = true;
          } catch (Exception e2) {
              try{
                 Thread.sleep(3000);
                 this.store = session.getStore();
                 this.store.connect(incomingServer, user, password);
                 isOpen = true;
              } catch (Exception e3) {
                 log.severe("Error Opening Connection Store: " + e3);
                 log.throwing(this.getClass().getName(), "openConnection", e3);
                 throw new EmailAccessException("Error Opening Connection Store (" + e3.getMessage() + ")");
              }
          }
      }
      */

      if (isOpen){
          try {
              this.mailDefaultFolder  = store.getDefaultFolder();
              this.defaultFolder      = df;
          } catch (Exception e) {
              log.severe("Error getting Folders: " + e);
              log.throwing(this.getClass().getName(), "openConnection", e);
              throw new EmailAccessException("Error getting Folders (" + e.getMessage() + ")");
          }
          log.info("IMAP Connection opened");
      } else {
          log.info("IMAP Error Opening Connection Store - Synchronization Interrupt");
      }

    }


    /**
     * closes the connection with the mail server
     *
     * @throws EmailAccessException
     */
    public void releaseConnection() throws EmailAccessException {
        try {
            this.store.close();
            this.store = null;
            this.session = null;
        } catch (Exception e) {
     	    throw new EmailAccessException("Error Closing Store " + e.toString());
        }

    }

    /**
     *
     * returns Inbox Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getInboxFolder() throws EmailAccessException {
       IMAPFolder inbox;
       try {
           inbox = (IMAPFolder)mailDefaultFolder.getFolder(
                                             defaultFolder.getInboxName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting Inbox Folder " +
                                          me.toString());
       }
       return inbox;
    }


    /**
     *
     * returns Outbox Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getOutboxFolder() throws EmailAccessException {
       IMAPFolder outbox;
       try {
           outbox = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getOutboxName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting outbox Folder " +
                                          me.toString());
       }
       return outbox;
    }


    /**
     *
     * returns Sent Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getSentFolder() throws EmailAccessException {
       IMAPFolder sent;
       try {
           sent = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getSentName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting sent Folder " +
                                          me.toString());
       }
       return sent;
    }

    /**
     *
     * returns Drafts Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getDraftsFolder() throws EmailAccessException {
       IMAPFolder drafts;
       try {
           drafts = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getDraftName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting drafts Folder " +
                                          me.toString());
       }
       return drafts;
    }

    /**
     *
     * returns Trash Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getTrashFolder() throws EmailAccessException {
       IMAPFolder trash;
       try {
           trash = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getTrashName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting trash Folder " +
                                          me.toString());
       }
       return trash;
    }

    //---------------------------------------------------------- PRIVATE METHODS

   /**
    *
    *
    * @param props Properties
    * @param outgoingServer String
    * @param outgoingPort String
    * @param incomingServer String
    * @param incomingPort String
    * @param provider Provider
    */
   protected void setConnectionSSL(Properties props,
                                   String outgoingServer,
                                   String outgoingPort,
                                   boolean smtpAuth,
                                   String incomingServer,
                                   String incomingPort,
                                   Provider provider,
                                   String keystorePath,
                                   String keystorePassphrase)
   {

        if (log.isLoggable(Level.FINEST)) {
           log.finest("IMAP set an SSL connection");
        }

        props.setProperty("mail.smtp.host" , outgoingServer);
        props.setProperty("mail.smtp.port" , outgoingPort);
        props.setProperty("mail.transport.protocol" , "smtp");

        if (smtpAuth){
           props.setProperty("mail.smtp.auth", "true");
        }

        props.setProperty("mail.imap.host" , incomingServer);
        props.setProperty("mail.imap.port" , incomingPort);

        props.setProperty("mail.store.protocol", "imap");

        System.setProperty("java.protocol.handler.pkgs",
                                  "com.sun.net.ssl.internal.www.protocol");
        System.setProperty("javax.net.ssl.trustStore", keystorePath);
       System.setProperty("javax.net.ssl.trustStorePassword", keystorePassphrase);

        provider = new Provider();
	    Security.addProvider(provider);

        // set SSL parameters
    	props.setProperty("mail.imap.socketFactory.class",
                                 "javax.net.ssl.SSLSocketFactory");
    	props.setProperty("mail.imap.socketFactory.fallback",
                                 "false");
    	props.setProperty("mail.imap.socketFactory.port",
                                 incomingPort);

    }

    /**
     *
     *
     *
     * @param props Properties
     * @param outgoingServer String
     * @param outgoingPort String
     * @param incomingServer String
     * @param incomingPort String
     */
    protected void setConnectionNormal(Properties props,
                                       String outgoingServer,
                                       String outgoingPort,
                                       boolean smtpAuth,
                                       String incomingServer,
                                       String incomingPort){

        if (log.isLoggable(Level.FINEST)) {
           log.finest("IMAP set a normal connection");
        }

        props.setProperty("mail.smtp.host", outgoingServer);
        props.setProperty("mail.smtp.port", outgoingPort);
        props.setProperty("mail.transport.protocol", "smtp");

        if (smtpAuth){
           props.setProperty("mail.smtp.auth", "true");
        }

        props.setProperty("mail.imap.host"    , incomingServer);
        props.setProperty("mail.imap.port"    , incomingPort);

        props.setProperty("mail.store.protocol", "imap");
    }


    // --------------------------------------------------------- PRIVATE METHODS

    /**
     *
     *
     */
    private boolean multipleOpenConnection(String incomingServer,
                                           String user,
                                           String password)
     throws EmailAccessException {

      boolean isOpen = false;
      String error = "";
      int islock   = -1;

      try {
          this.store = session.getStore();
          this.store.connect(incomingServer, user, password);
          isOpen = true;
      } catch (Exception e1) {
          error = e1.getMessage();
          islock = error.indexOf("lock");
          if (islock != -1){
              try {
                 Thread.sleep(3000);
                 this.store = session.getStore();
                 this.store.connect(incomingServer, user, password);
                 isOpen = true;
              } catch (Exception e2) {
                  error = e1.getMessage();
                  islock = error.indexOf("lock");
                  if (islock != -1){
                      try {
                         Thread.sleep(3000);
                         this.store = session.getStore();
                         this.store.connect(incomingServer, user, password);
                         isOpen = true;
                      } catch (Exception e3) {
                         log.severe("Error Opening Connection Store: " + e3);
                         log.throwing(this.getClass().getName(), "openConnection", e3);
                         throw new EmailAccessException("Error Opening Connection Store (" + e3.getMessage() + ")");
                      }
                  } else {
                     log.severe("Error Opening Connection Store: " + e2);
                     log.throwing(this.getClass().getName(), "openConnection", e2);
                     throw new EmailAccessException("Error Opening Connection Store (" + e2.getMessage() + ")");
                  }
              }
          } else {
              log.severe("Error Opening Connection Store: " + e1);
              log.throwing(this.getClass().getName(), "openConnection", e1);
              throw new EmailAccessException("Error Opening Connection Store (" + e1.getMessage() + ")");
          }
      }

      return isOpen;

    }


}
